#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <zlib.h>
//#include "banded.h"

#define LNAME 256
#define LSEQ 32768
#define LBUF (3*LSEQ)
#define LAUX 32

int NUM_ALIGNMENTS = 256;

int maxeval = 1.0;

typedef struct cigar_t{
  int num;
  char sym;
}cigar_t;

typedef struct cmaux_t{
  char sym;
  int num;
}cmaux_t;

typedef struct sam_t{
  char * qname;
  int flag;
  char * rname;
  int pos;
  int mapq;
  struct cigar_t * cigar;
  int cigar_length;
  char * rnext;
  int pnext;
  int tlen;
  char * seq;
  char * qual;
  double as;// bit score
  double ev;// e-value
  double pident;// percentage of identical matches
}sam_t;

typedef struct region{
  int left;
  int right;
}region_t;

typedef struct blast{
  int sstart,send,qstart,qend;
  double bitscore,evalue,pident;
}blast_t;

void print_sam(sam_t * s){
  printf("%s",s->qname);
  printf("\t");
  printf("%d",s->flag);
  printf("\t");
  printf("%s",s->rname);
  printf("\t");
  printf("%d",s->pos);
  printf("\t");
  printf("%d",s->mapq);
  printf("\t");
  int i;
  for(i=0; i<s->cigar_length; ++i){
    printf("%d%c",s->cigar[i].num,s->cigar[i].sym);
  }
  //printf("#%d#",s->cigar_length);
  printf("\t");
  printf("%s",s->rnext);
  printf("\t");
  printf("%d",s->pnext);
  printf("\t");
  printf("%d",s->tlen);
  printf("\t");
  printf("%s",s->seq);
  printf("\t");
  printf("%s",s->qual);
  printf("\t");
  printf("AS:i:%d",(int)(s->as+0.499));// sude ni shisha go nyu shite ari soh nanode ? 
  //printf("%s",s->as);
  printf("\t");
  if(s->ev > 0.0){
    printf("EV:Z:%1.0e",s->ev);
  }
  else{
    printf("EV:Z:%1.1f",s->ev);
  }
  //printf("%s",s->ev);
  printf("\t");
  printf("PI:Z:%2.2f",s->pident);
  printf("\n");
  return;
}

void init_sam(sam_t * s){
  s->qname = (char*)malloc(sizeof(char)*LNAME);
  if(s->qname == NULL){
    fprintf(stderr,"cannot allocate memory: qname\n");
    abort();
  }
  s->rname = (char*)malloc(sizeof(char)*LNAME);
  if(s->rname == NULL){
    fprintf(stderr,"cannot allocate memory: rname\n");
    abort();
  }
  s->cigar = (cigar_t*)malloc(sizeof(cigar_t)*LSEQ);
  if(s->cigar == NULL){
    fprintf(stderr,"cannot allocate memory: cigar\n");
    abort();
  }
  s->rnext = (char*)malloc(sizeof(char)*LNAME);
  if(s->rnext == NULL){
    fprintf(stderr,"cannot allocate memory: rnext\n");
    abort();
  }
  s->seq = (char*)malloc(sizeof(char)*LSEQ);
  if(s->seq == NULL){
    fprintf(stderr,"cannot allocate memory: seq\n");
    abort();
  }
  s->qual = (char*)malloc(sizeof(char)*LSEQ);
  if(s->qual == NULL){
    fprintf(stderr,"cannot allocate memory: qual\n");
    abort();
  }
  return;
}

void reset_cigar(sam_t * s){
  /*
  int i;
  for(i=0; i<LSEQ; ++i){
    s->cigar[i].num = 0;
    s->cigar[i].sym = '\0';
  }
  */
  s->cigar_length=0;
}

void reset_sam(sam_t * s){
  s->qname[0] = '\0';
  s->rname[0] = '\0';
  s->rnext[0] = '\0';
  s->cigar_length=0;
  s->seq[0] = '\0';
  s->qual[0] = '\0';
  return;
}

void free_sam(sam_t * s){
  free(s->qname);
  free(s->rname);
  free(s->cigar);
  free(s->rnext);
  free(s->seq);
  free(s->qual);
  return;
}

void blast_print_sam(sam_t * sam, cmaux_t * cmaux, blast_t * blast, char** quals);
void aln2cm(sam_t * sam, char * q, char * s, cmaux_t * cmaux);

int opt_invsam=1;
int opt_unique_ref=0;
double valid_evalue=-1.0;
int opt_chimeric_filter=0;
//int opt_banded_realignment=0;
int opt_fq=0;
int num_orig_records=0;
int opt_dummy_qv=0;


char fqname[1024];

int main(int argc, char ** argv)
{

  int hitnum=0;
  {
    int result;
    while((result=getopt(argc,argv,"iue:q:c:r:d")) != -1){
    //while((result=getopt(argc,argv,"iue:cb")) != -1)
      switch(result){
        case 'i':
          opt_invsam=1;
          ++hitnum;
          break;
        case 'u':
          opt_unique_ref=1;
          ++hitnum;
          break;
        case 'e':
          valid_evalue=atof(optarg);
          if(valid_evalue < 0.0){
            fprintf(stderr,"e>=0.0: %lf is given.",valid_evalue);
            abort();
          }
          hitnum+=2;
          break;
        case 'q':
          opt_fq=1;
          if(strlen(optarg) < 1024){
            strcpy(fqname,optarg);
          }
          else{
            fprintf(stderr,"too long fastq file name\n");
            abort();
          }
          hitnum+=2;
          break;
        case 'c':
          opt_chimeric_filter=atoi(optarg);
          if(opt_chimeric_filter < 0){
            fprintf(stderr,"c: %d is given.",opt_chimeric_filter);
            fprintf(stderr,"\tmust be >= 0");
            abort();
          }
          hitnum+=2;
          break;
        //case 'b':
        //  opt_banded_realignment=1;
        //  ++hitnum;
        //  break;
        case 'r':
          num_orig_records=atoi(optarg);
          if(num_orig_records < 1){
            fprintf(stderr,"n: %d is given.",num_orig_records);
            abort();
          }
          hitnum+=2;
          break;
        case 'd':
          opt_dummy_qv=1;
          ++hitnum;
          break;
        case '?':
          printf("humei\n");
          break;
        default:
          break;
      }
    }
    //argc -= hitnum;
  }

  if(argc != 2+hitnum){
    fprintf(stderr, "USAGE: <this> -q <cXX.all.idfq.gz> <in.blastn | - >\n");
    fprintf(stderr, "\t-q <cXX.all.idfq[.gz]> : embeds quality values\n");
    char msg[] = "off";
    if(opt_invsam){
      strcpy(msg,"on");
    }
    fprintf(stderr, "\t-i: regards blasted queries as references in the output sam (default: %s)\n",msg);
    fprintf(stderr, "\t-u: avoids double voting\n");
    fprintf(stderr, "\t-e <valid_evalue_threshold> : discards input records with more than the threshold\n");
    fprintf(stderr, "\t-d: give dummy qvs to save memory\n");
    return 1;
  }
  char * in_blastn = argv[1+hitnum];

  //double ev;
  /*
  char * q;
  char * ali;
  char * s;
  q = (char*)malloc(sizeof(char)*LSEQ);
  if(q == NULL){
    fprintf(stderr,"cannot allocate memory: q\n");
    abort();
  }
  ali = (char*)malloc(sizeof(char)*LSEQ);
  if(ali == NULL){
    fprintf(stderr,"cannot allocate memory: ali\n");
    abort();
  }
  s = (char*)malloc(sizeof(char)*LSEQ);
  if(s == NULL){
    fprintf(stderr,"cannot allocate memory: s\n");
    abort();
  }
  */
  char * prev_ref_name = (char*)malloc(sizeof(char)*LNAME);
  if(prev_ref_name == NULL){
    fprintf(stderr,"cannot allocate memory: prev_ref_name\n");
    abort();
  }
  prev_ref_name[0] = '\0';
  //ev = maxeval+1.0;
  sam_t sam;
  init_sam(&sam);
  reset_sam(&sam);
  sam.qname[0]='\0';
  sam.mapq = 255;
  sam.rnext[0]='*';
  sam.rnext[1]='\0';
  sam.pnext=0;
  sam.tlen=0;
  char * query_name = (char*)malloc(sizeof(char)*LNAME);
  if(query_name == NULL){
    fprintf(stderr,"cannot allocate memory: query_name\n");
    abort();
  }
  FILE * fp;
  if(in_blastn[0] == '-'){
    fp = stdin;
  }
  else{
    fp = fopen(in_blastn,"r");
  }
  if(fp == NULL){
    fprintf(stderr,"cannot open the file %s\n", in_blastn);
    abort();
  }
  char * buf = (char*)malloc(sizeof(char)*LBUF);
  if(buf == NULL){
    fprintf(stderr, "cannot allocate memory: buf\n");
    abort();
  }
  char * bufq = (char*)malloc(sizeof(char)*LSEQ);
  if(bufq == NULL){
    fprintf(stderr, "cannot allocate memory: bufq\n");
    abort();
  }
  region_t * regions = (region_t*)malloc(sizeof(region_t)*NUM_ALIGNMENTS);
  if(regions == NULL){
    fprintf(stderr, "cannot allocate memory: regions\n");
    abort();
  }
  int region_index=0;
  int scanfret;
  //int sstart,send,qstart,qend;
  //double bitscore,evalue,pident;
  blast_t blast;

  // FIXME
  // We assume if the given file fqname is in plain text, gzopen works as fopen.
  // Tell us the source.
  // TODO
  gzFile gzfq;
  int fq_records=0;
  int maxreadlength=LSEQ;
  char ** quals;
  if(opt_dummy_qv == 0 && opt_fq){
    gzfq = gzopen(fqname, "rb");
    if(gzfq == Z_NULL){
      fprintf(stderr, "cannot open the file fqname\n");
      abort();
    }
    if(num_orig_records == 0){
      while(gzgets(gzfq,buf,LBUF) != Z_NULL){
        if(buf[0] == '@'){// qual may start from '@' XXX
          ++fq_records;
          gzgets(gzfq,buf,LBUF);
          gzgets(gzfq,buf,LBUF);
          gzgets(gzfq,buf,LBUF);
        }
        else{
          fprintf(stderr, "FATAL: unexpected fastq\n");
          fprintf(stderr, "we expected:\n");
          fprintf(stderr, "@id\n");
          fprintf(stderr, "nucleotides in one line\n");
          fprintf(stderr, "+[something]\n");
          fprintf(stderr, "quality values in one line\n");
          abort();
        }
      }
      gzseek(gzfq,0,SEEK_SET);
      fq_records = fq_records*2;
    }
    else{
      fq_records = num_orig_records;
    }
    quals = (char**)malloc(sizeof(char*)*fq_records);
    if(quals == NULL){
      fprintf(stderr, "cannot allocate memory: quals\n");
      abort();
    }
    {
      int i;
      for(i=0; i<fq_records; ++i){
        quals[i] = (char*)malloc(sizeof(char)*maxreadlength);
        if(quals[i] == NULL){
          fprintf(stderr,"cannot allocate memory: quals[%d]\n",i);
          abort();
        }
      }
    }
    while(gzgets(gzfq,buf,LBUF) != Z_NULL){
      int id;// 1-origin
      sscanf(buf,"@%d",&id);
      --id;
      if(id<0){
        fprintf(stderr, "id must be 1-origin. given id is %d\n",id+1);
        fprintf(stderr, "%s\n",buf);
        abort();
      }
      if(id+1 > fq_records){
        fq_records = fq_records * 2;
        char ** tmp;
        tmp = (char**)realloc(quals,sizeof(char*)*fq_records);
        if(tmp == NULL){
          fprintf(stderr, "cannot reallocate memory: quals. fq_records %d\n",fq_records);
          abort();
        }
        else{
          quals = tmp;
          {
            int i;
            for(i=fq_records/2; i<fq_records; ++i){
              quals[i] = (char*)malloc(sizeof(char)*maxreadlength);
              if(quals[i] == NULL){
                fprintf(stderr,"cannot allocate memory: quals[%d]\n",i);
                abort();
              }
            }
          }
        }
      }
      gzgets(gzfq,buf,LBUF);//bases
      gzgets(gzfq,buf,LBUF);//+
      gzgets(gzfq,quals[id],maxreadlength);//qvs
      int len = strlen(quals[id]);
      if(quals[id][len-1] == '\n'){
        quals[id][len-1] = '\0';
      }
    }
    gzclose(gzfq);
  }
  /*
  {
    int i;
    for(i=0; i<fq_records; ++i){
      printf("@%d\n",i);
      printf("%s\n",quals[i]);
    }
  }
  return 0;
  */
    
//  int * score_matrix;
//  int * tb_matrix;
//  char * r_txt;
//  char * r_pat;
//  char * r_ali;
//  if(opt_banded_realignment==1){
//    init_banded_variables(&score_matrix, &tb_matrix, &r_txt, &r_pat, &r_ali, LSEQ,(int)(LSEQ*0.2));
//  }

  while(fgets(buf,LBUF,fp) != NULL){
    if(buf[0] == '#'){
      prev_ref_name[0] = '\0';
      region_index=0;
      continue;
    }
    // -outfmt '7 qseqid sstart send sacc qstart qend bitscore evalue pident qseq sseq'
    scanfret = sscanf(buf,"%s %d %d %s %d %d %lf %lf %lf %s %s", sam.qname, &blast.sstart, &blast.send, sam.rname, &blast.qstart, &blast.qend, &blast.bitscore, &blast.evalue, &blast.pident, sam.seq, sam.qual);// sam.qual <- sseq
    //scanfret = sscanf(buf,"%s %d %d %s %d %d %lf %lf %lf %s %s", sam.qname, &sstart, &send, sam.rname, &qstart, &qend, &bitscore, &evalue, &pident, sam.seq, sam.qual);
    if(scanfret != 11){
      fprintf(stderr, "sth strange: scanfret %d\n",scanfret);
      abort();
    }
    if(blast.qend-blast.qstart < 0){
      fprintf(stderr, "souteigai blast qstt qend %d %d\n",blast.qstart,blast.qend);
      abort();
    }
    if(valid_evalue >= 0.0){
      if(blast.evalue > valid_evalue){
        continue;
      }
    }
    //opt_unique_ref=1;
    if(opt_unique_ref == 1 && strcmp(prev_ref_name, sam.rname)==0){
      // check overlap
      int i=0;
      int cf=0;
      for(i=0; i<region_index; ++i){
        if(regions[i].left <= blast.sstart && blast.sstart <= regions[i].right){
          cf=1;
          break;
        }
        if(regions[i].left <= blast.send && blast.send <= regions[i].right){
          cf=1;
          break;
        }
      }
      if(cf==1){
        continue;
      }
    }
    else{
      // another ref
      region_index=0;
      strcpy(prev_ref_name, sam.rname);
    }
    if(blast.send - blast.sstart < 0){
      // reverse
      sam.flag = 0x10;
      sam.pos = blast.send;
      regions[region_index].left=blast.send;
      regions[region_index].right=blast.sstart;
      ++region_index;
    }
    else{
      // forward
      sam.flag = 0x0;
      sam.pos = blast.sstart;
      regions[region_index].left=blast.sstart;
      regions[region_index].right=blast.send;
      ++region_index;
    }
    if(region_index >= NUM_ALIGNMENTS){
      NUM_ALIGNMENTS = NUM_ALIGNMENTS * 2;
      region_t * tmp = (region_t*)realloc(regions,sizeof(region_t)*NUM_ALIGNMENTS);
      if(tmp == NULL){
        fprintf(stderr, "cannot reallocate memory: regions. NUM_ALIGNMENTS %d\n",NUM_ALIGNMENTS);
        abort();
      }
      else{
        regions = tmp;
      }
    }
    if(abs(blast.bitscore) > 255.0){
      sam.mapq = 255;
    }
    else{
      sam.mapq = (int)(abs(blast.bitscore)+0.499);
    }
    cmaux_t cmaux;
    cmaux.sym='\0'; cmaux.num=0;
    sam.cigar_length=0;

//    if(opt_banded_realignment==1){
//      delete_hyphen(sam.qual);
//      delete_hyphen(sam.seq);
//      int ret_banded=banded(sam.qual, sam.seq, r_txt, r_pat, r_ali,(int)(strlen(sam.qual)*0.2), score_matrix, tb_matrix);
//      //fprintf(stderr, "%s\n",r_txt);
//      //fprintf(stderr, "%s\n",r_ali);
//      //fprintf(stderr, "%s\n",r_pat);
//      if(ret_banded != 0){
//        continue;
//      }
//      strcpy(sam.qual, r_txt);
//      strcpy(sam.seq, r_pat);
//    }
    // trim terminals agains chimera
    if(opt_chimeric_filter > 0){
      if(strcmp(sam.qname,sam.rname) != 0){
        int trim = opt_chimeric_filter;
        int len_sseq = strlen(sam.qual);
        if(len_sseq != strlen(sam.seq)){
          fprintf(stderr, "lengths of que and sbj differ\n");
          abort();
        }
        if(len_sseq <= trim*2){// sam.qual <- sseq
          continue;
        }
        int h_trim = trim;
        int t_trim = trim;

        while(sam.seq[h_trim] == '-' || sam.qual[h_trim] == '-'){
          ++h_trim;
        }
        while(sam.seq[len_sseq-t_trim] == '-' || sam.qual[len_sseq-t_trim] == '-'){
          ++t_trim;
        }
        if(h_trim > len_sseq-t_trim){
          continue;
        }

        // adapt blast.qstart
        {
          int nhyphen=0;
          int i;
          for(i=0; i<h_trim; ++i){
            if(sam.seq[i] == '-')
              ++nhyphen;
          }
          blast.qstart += (h_trim-nhyphen);
        }

        // adapt blast.qend
        {
          int nhyphen=0;
          int i;
          int endpos=strlen(sam.seq)-1;
          for(i=0; i<t_trim; ++i){
            if(sam.seq[endpos-i] == '-')
              ++nhyphen;
          }
          blast.qend -= (t_trim-nhyphen);
        }
        
        // adapt blast.sstart
        {
          int nhyphen=0;
          int i;
          for(i=0; i<h_trim; ++i){
            if(sam.qual[i] == '-')
              ++nhyphen;
          }
          if(sam.flag == 0x0){
            blast.sstart += (h_trim-nhyphen);
          }
          else if(sam.flag == 0x10){
            blast.sstart -= (h_trim-nhyphen);
          }
        }

        // adapt blast.send
        {
          int nhyphen=0;
          int i;
          int endpos=strlen(sam.qual)-1;
          for(i=0; i<t_trim; ++i){
            if(sam.qual[endpos-i] == '-')
              ++nhyphen;
          }
          if(sam.flag == 0x0){
            blast.send -= (t_trim-nhyphen);
          }
          else if(sam.flag == 0x10){
            blast.send += (t_trim-nhyphen);
          }
        }

        // adapt sam.pos
        if(sam.flag == 0x0){
          int nhyphen=0;
          int i;
          for(i=0; i<h_trim; ++i){
            if(sam.qual[i] == '-')
              ++nhyphen;
          }
          sam.pos += (h_trim-nhyphen);
        }
        else if(sam.flag == 0x10){
          int nhyphen=0;
          int i;
          int endpos=strlen(sam.qual)-1;
          for(i=0; i<t_trim; ++i){
            if(sam.qual[endpos-i] == '-')
              ++nhyphen;
          }
          sam.pos += (t_trim-nhyphen);
        }
        else{
          fprintf(stderr, "souteigai flag\n");
          abort();
        }

        strcpy(bufq, &sam.qual[h_trim]);
        bufq[strlen(bufq)-t_trim]='\0';
        strcpy(sam.qual, bufq);
        strcpy(bufq, &sam.seq[h_trim]);
        bufq[strlen(bufq)-t_trim]='\0';
        strcpy(sam.seq, bufq);

        //strcpy(sam.qual, &r_txt[42]);
        //strcpy(sam.seq, &r_pat[42]);
        //sam.qual[strlen(sam.qual)-42]='\0';
        //sam.seq[strlen(sam.seq)-42]='\0';

//        sam.pos += trim;
//        blast.qend -= trim;
//        if(blast.send - blast.sstart < 0){
//          blast.sstart -= trim;
//          blast.send += trim;
//        }
//        else{
//          blast.send -= trim;
//          blast.sstart += trim;
//        }
      }
      else{
        // ref
        // do nothing
      }
    }
    /*
    else{
      // obsolete
      strcpy(sam.qual, r_txt);
      strcpy(sam.seq, r_pat);
    }
    */

    aln2cm(&sam, sam.seq, sam.qual, &cmaux);

    sam.rnext[0] = '*';
    sam.rnext[1] = '\0';
    sam.pnext=0;
    sam.tlen=0;
    sam.as=abs(blast.bitscore);
    sam.ev=blast.evalue;
    sam.pident=blast.pident;
    /*
     * unable to know the number of hard clipping, cos fmt 7 does'nt show query length
    if(qstart > 1){
      sam.cigar[sam.cigar_length].sym = 'H';
      sam.cigar[sam.cigar_length].num = qstart-1;
      ++sam.cigar_length;
    }
    */
    blast_print_sam(&sam, &cmaux, &blast, quals);
  }

  if(in_blastn[0] != '-'){
    fclose(fp);
  }
  free_sam(&sam);
  //free(q);
  //free(ali);
  //free(s);
  free(query_name);
  free(buf);
  free(bufq);
  free(prev_ref_name);
  if(opt_dummy_qv == 0 && opt_fq)
  {
    int i;
    for(i=0; i<fq_records; ++i){
      free(quals[i]);
    }
    free(quals);
  }
  free(regions);
//  if(opt_banded_realignment==1){
//    free(score_matrix);
//    free(tb_matrix);
//    free(r_txt);
//    free(r_pat);
//    free(r_ali);
//  }
  return 0;
}

void blast_print_sam(sam_t * sam, cmaux_t * cmaux, blast_t * blast, char** quals){
  if(cmaux->num > 0){
    sam->cigar[sam->cigar_length].sym = cmaux->sym;
    sam->cigar[sam->cigar_length].num = cmaux->num;
    ++sam->cigar_length;
  }

  if(opt_invsam == 1){
    char * tmp = sam->rname;
    sam->rname = sam->qname;
    sam->qname = tmp;
    // sam->flag = sam->flag
    sam->pos = blast->qstart;
    // sam->mapq = sam->mapq
    {
      int i;
      int loop;
      for(i=0,loop=sam->cigar_length; i<loop; ++i){
        if(sam->cigar[i].sym == 'D'){
          sam->cigar[i].sym = 'I';
        }
        else if(sam->cigar[i].sym == 'I'){
          sam->cigar[i].sym = 'D';
        }
        else if(sam->cigar[i].sym == 'M'){
          // do nothing
        }
        else{
          fprintf(stderr, "souteigai cigar\n");
          abort();
        }
      }
    }
    // swap query for subject
    tmp = sam->seq;
    sam->seq = sam->qual;
    sam->qual = tmp;
  }

  {
    int loop=strlen(sam->seq);
    int i,j;
    for(i=0,j=0; i<loop; ++i){
      if(sam->seq[i] != '-'){
        sam->qual[j++] = sam->seq[i];
      }
    }
    sam->qual[j]='\0';
    char * tmp = sam->seq;
    sam->seq = sam->qual;
    sam->qual = tmp;
  }

  // append quality values
  sam->qual[0] = '*';
  sam->qual[1] = '\0';
  if(opt_fq)
  {
    int id;
    sscanf(sam->qname,"%d",&id);
    --id;
    if(id<0){
      fprintf(stderr,"id must be 1-origin\n");
      abort();
    }

    int stt;// 1-origin
    int end;// 1-origin
    if(opt_invsam){
      stt = blast->sstart;
      end = blast->send;
    }
    else{
      stt = blast->qstart;
      end = blast->qend;
    }

    if(stt <= end){
      int len = end-stt+1;
      // TODO use sql or sth that shares memory
      if(opt_dummy_qv){
        int i;
        for(i=0; i<len; ++i){
          sam->qual[i] = '5';
        }
        sam->qual[len] = '\0';
      }
      else{
        strncpy(sam->qual, &quals[id][stt-1], len);
        sam->qual[len] = '\0';
      }
      if(blast->sstart > blast->send){
        int i;
        for(i=0; i<len/2; ++i){
          char tmp = sam->qual[i];
          sam->qual[i] = sam->qual[len-1-i];
          sam->qual[len-1-i] = tmp;
        }
      }
    }
    else{// opt_invsam && blast->sstart > blast->send
      int len = stt-end+1;
      if(opt_dummy_qv){
        int i;
        for(i=0; i<len; ++i){
          sam->qual[i] = '5';
        }
        sam->qual[len] = '\0';
      }
      else{
        strncpy(sam->qual, &quals[id][end-1], len);
        sam->qual[len] = '\0';
      }
      len = strlen(sam->qual);
      int i;
      for(i=0; i<len/2; ++i){
        char tmp = sam->qual[i];
        sam->qual[i] = sam->qual[len-1-i];
        sam->qual[len-1-i] = tmp;
      }
    }
  }

  /*
  if(qrest){
    sam->cigar[sam->cigar_length].sym = 'H';
    sam->cigar[sam->cigar_length].num = qrest;
    ++sam->cigar_length;
  }
  */
  if(opt_invsam != 1 && sam->flag & 0x10){
    {
      int i;
      int loop;
      int len;
      for(i=0,len=sam->cigar_length,loop=len/2; i<loop; ++i){
        char tmp = sam->cigar[i].sym;
        int tmp2= sam->cigar[i].num;
        sam->cigar[i].sym = sam->cigar[len-1-i].sym;
        sam->cigar[i].num = sam->cigar[len-1-i].num;
        sam->cigar[len-1-i].sym = tmp;
        sam->cigar[len-1-i].num = tmp2;
      }
//      for(i=0,len=strlen(sam->qual),loop=len/2; i<loop; ++i){
//        char tmp = sam->qual[i];
//        sam->qual[i] = sam->qual[len-1-i];
//        sam->qual[len-1-i] = tmp;
//      }
      for(i=0,len=strlen(sam->seq),loop=len/2; i<loop; ++i){
        char tmp = sam->seq[i];
        sam->seq[i] = sam->seq[len-1-i];
        sam->seq[len-1-i] = tmp;
      }
      for(i=0,loop=strlen(sam->seq); i<loop; ++i){
        char t = sam->seq[i];
        switch(t){
          case 'a':
            t = 'T';
            break;
          case 'c':
            t = 'G';
            break;
          case 'g':
            t = 'C';
            break;
          case 't':
            t = 'A';
            break;
          case 'A':
            t = 'T';
            break;
          case 'C':
            t = 'G';
            break;
          case 'G':
            t = 'C';
            break;
          case 'T':
            t = 'A';
            break;
          default:
            fprintf(stderr, "souteigai : %c\n",t);
            abort();
            break;
        }
        sam->seq[i] = t;
      }
    }
  }
  print_sam(sam);
  return;
}

void aln2cm(sam_t * sam, char * q, char * s, cmaux_t * cmaux){
  int len = strlen(q);
  int i;
  for(i=0; i<len; ++i){
    int op;
    if(q[i] == '-'){
      op = 'D';
    }
    else if(s[i] == '-'){
      op = 'I';
    }
    else{
      op = 'M';
    }
    if(cmaux->sym == op){
      ++cmaux->num;
    }
    else{
      if(cmaux->num > 0){
        sam->cigar[sam->cigar_length].sym = cmaux->sym;
        sam->cigar[sam->cigar_length].num = cmaux->num;
        ++sam->cigar_length;
      }
      cmaux->sym = op;
      cmaux->num = 1;
    }
  }
  return;
}
