#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;

my $DEVEL;

my $from=0;
my $to=1;
my $fastqdir="./";
my $ca_path="";

GetOptions("from=i" => \$from, "to=i" => \$to,"d=s" => \$fastqdir,"devel" => \$DEVEL,'ca_path=s'=>\$ca_path);

if(@ARGV != 2){
  die "USAGE: <this> <asm.spec> estimated_genome_size [-from integer -to integer]\n\t[-d directory in which fin.fq.gzs exist]\n\t[-ca_path /path/to/your/wgs/Linux-amd64/bin]\n";
}
#my $spec = "myasm7.1.spec";
#my $spec = "myasm7.spec";
my $spec = $ARGV[0];
printf STDERR ("%s is given\n",$spec);
my $estimated_genome_size = $ARGV[1];
if($estimated_genome_size <= 0){
  die "estimated_genome_size must be > 0\n";
}

printf STDERR ("#>- params -<#\n");
printf STDERR ("spec\t%s\n",$spec);
printf STDERR ("estimated_genome_size\t%s\n",$estimated_genome_size);
printf STDERR ("from\t%s\n",$from);
printf STDERR ("to\t%s\n",$to);
printf STDERR ("fastq_dir\t%s\n",$fastqdir);
if($DEVEL){
  printf STDERR ("development mode\t%s\n","true");
}
if($ca_path){
  printf STDERR ("ca_path\t%s\n",$ca_path);
}
printf STDERR ("#>- params -<#\n");
#exit;

my $bashcommand="";

my $suffix = "top20x";

printf STDERR ("start idfq2fq\n");

for(my $i=$from+1; $i<=$to; ++$i){
  my $prefix=sprintf("c%02d.fin",$i);
  my $PG1 = "get_top_20x_fa.pl";
  if($DEVEL){
    $PG1 = "./$PG1";
  }
  $bashcommand .= "(gzip -d -c $fastqdir/$prefix.idfq.gz | $PG1 - -g $estimated_genome_size -q > $prefix.$suffix.fq) &\n";
}

`
  $bashcommand
  wait
`;
printf STDERR ("done idfq2fq\n");

printf STDERR ("start fastqToCA\n");
for(my $i=$from+1; $i<=$to; ++$i){
  my $prefix=sprintf("c%02d.fin",$i);
  my $fastqToCA = "fastqToCA";
  if($ca_path){
    $fastqToCA = "$ca_path/$fastqToCA";
  }
  `$fastqToCA -libraryname foo -technology pacbio -reads $prefix.$suffix.fq > $prefix.$suffix.frg`;
}
printf STDERR ("done fastqToCA\n");

printf STDERR ("start CA (stopAfter=unitigger)\n");

my $now = `date +%Y%m%d%H%M%S`;
chomp $now;
my $now_used=0;

my $runCA="runCA";
if($ca_path){
  $runCA = "$ca_path/$runCA";
}

for(my $i=$from+1; $i<=$to; ++$i){
  my $prefix=sprintf("c%02d.fin",$i);
  #`bin/runCA -dir $prefix -p $prefix -s $spec $prefix.frg |& tee do_$prefix.log`;
  if(-e "$prefix.$suffix"){
    #$prefix = $now."_".$prefix;
    $now_used=1;
    `$runCA stopAfter=unitigger -dir ${now}_$prefix.$suffix -p ${now}_$prefix.$suffix -s $spec $prefix.$suffix.frg |& tee -a do_${now}_$prefix.$suffix.log`;
  }
  else{
    `$runCA stopAfter=unitigger -dir $prefix.$suffix -p $prefix.$suffix -s $spec $prefix.$suffix.frg |& tee -a do_$prefix.$suffix.log`;
  }
}
printf STDERR ("done CA (stopAfter=unitigger)\n");

printf STDERR ("start CA (the rest)\n");
$bashcommand="";
for(my $i=$from+1; $i<=$to; ++$i){
  my $prefix=sprintf("c%02d.fin",$i);
  if($now_used==1){
    #$prefix = $now."_".$prefix;
    $bashcommand .= "($runCA -dir ${now}_$prefix.$suffix -p ${now}_$prefix.$suffix -s $spec $prefix.$suffix.frg |& tee -a do_${now}_$prefix.$suffix.log) &\n";
  }
  else{
    $bashcommand .= "($runCA -dir $prefix.$suffix -p $prefix.$suffix -s $spec $prefix.$suffix.frg |& tee -a do_$prefix.$suffix.log) &\n";
  }
}
`
  $bashcommand
`;
