#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;

my $VALIDDEPTH=1;
my $VALIDLENGTH=1;
my $fasta=0;
my $finish=0;
my $opt_dfq_check=0;

GetOptions('valid_depth=i' => \$VALIDDEPTH, 'valid_read_length=i' => \$VALIDLENGTH,
'f'=>\$fasta,'finish'=>\$finish,'check'=>\$opt_dfq_check);

if(@ARGV != 1){
  die "USAGE: <this> <in.dfq>\n\t[-f (outputs in fasta)\n\t --valid_depth int\n\t --valid_read_length int\n\t --finish (chops low depth (<valid_depth) regions)\n\t --check (outputs right dfq records and discard the rest)]\n";
}

if($VALIDDEPTH <= 0){
  $VALIDDEPTH = 1;
}
if($VALIDLENGTH <= 0){
  $VALIDLENGTH = 1;
}

my $counter=0;
my $printed_line=0;

my $line=<>;
++$counter;
my $result;

while(!eof){
  chomp $line;
  $result = $line =~ s/^\@//;
  if(!$result){
    die "1. strange input\n";
  }
  my $chr = $line;
  ++$printed_line;

  my $consensus = "";
  $line=<>;
  my $line_c = 1;
  chomp $line;
  while(1){# read bases
    $consensus.=$line;
    if(eof){
      die "2. strange input\n";
    }
    $line=<>;
    chomp $line;
    if($line =~ /^\+/){
      last;
    }
    else{
      ++$line_c;
    }
  }
  ++$printed_line;
  ++$printed_line;

  my $depths="";
  if(!$fasta){
    chomp $line;
    $depths = (split /\t/, $line)[1];
  }
  my $qvs = "";
  for(my $i=0; $i<$line_c; ++$i){# # of lines of bases and qvs must be =
    if(eof){
      last;
    }
    $line=<>;
    chomp $line;
    $qvs.=$line;
  }
  ++$printed_line;

  if(!$fasta){
    if($finish){
      &print_bases(\$chr,\$consensus,\$depths,\$qvs);
    }
    elsif($opt_dfq_check){
      if(length($consensus) == length($depths) && length($depths) == length($qvs)){
        printf("\@%s\n",$chr);
        printf("%s\n",$consensus);
        printf("+\t%s\n",$depths);
        printf("%s\n",$qvs);
      }
      else{
        die "broken record\n";
      }
    }
    else{
      &flush(\$chr,\$consensus,\$qvs,\$depths);
    }
  }
  else{
    if(length($consensus) >= $VALIDLENGTH){
      # writes in fasta
      $consensus =~ s/\-//g;
      printf(">%s\n",$chr);
      printf("%s\n",$consensus);
    }
  }

  if(!eof){
    $line = <>;# next unit's name
  }
  ++$counter;
}

if(!eof){
  printf STDERR "strange fastq\n";
}
if($printed_line%4 !=0){
  printf STDERR ("strange fastq: the number of lines is $printed_line, $printed_line % 4 = %d\n",$printed_line%4);
}

sub print_bases($$$$){
  my ($chr,$consensus,$depths,$qvs)=@_;
  my $bases="";
  my $qvs_tbp="";
  my $part=0;
  my $loop = length($$depths);
  for(my $i=0; $i<$loop; ++$i){
    if(ord(substr($$depths,$i,1))-33 >= $VALIDDEPTH){
      $bases .= substr($$consensus,$i,1);
      $qvs_tbp .= substr($$qvs,$i,1);
    }
    else{
      &flush($chr,\$bases,\$qvs_tbp,\$part);
      $bases="";
      $qvs_tbp="";
    }
  }
  &flush($chr,\$bases,\$qvs_tbp,\$part);
  $bases="";
  $qvs_tbp="";
}

sub flush($$$$){
  my($chr,$bases,$qvs,$part) = @_;
  if(length($$bases) != length($$qvs)){
    die "FATAL: bases and qvs have different lengths\n";
  }
  #$$bases =~ s/\-//g;
  {
    my $tmp_bases="";
    my $tmp_qvs="";
    my $tmp_depths="";
    my $loop = length($$bases);
    for(my $i=0; $i<$loop; ++$i){
      my $base = substr($$bases,$i,1);
      my $qv = substr($$qvs,$i,1);
      my $depth;
      if(!$finish){
        $depth = substr($$part,$i,1);#depth
      }
      if($base eq '-'){
        next;
      }
      $tmp_bases .= $base;
      $tmp_qvs .= $qv;
      if(!$finish){
        $tmp_depths .= $depth;
      }
    }
    $$bases = $tmp_bases;
    $$qvs = $tmp_qvs;
    if(!$finish){
      $$part = $tmp_depths;
    }
  }

  $$bases =~ tr/a-z/A-Z/;
  if(length($$bases)>= $VALIDLENGTH){
    if($finish){
      printf("\@%s/%06d\n",$$chr,$$part++);
      printf("%s\n",$$bases);
      printf("+\n");
      printf("%s\n",$$qvs);
    }
    else{
      printf("\@%s\n",$$chr);
      printf("%s\n",$$bases);
      printf("+\t%s\n",$$part);# part <- depths
      printf("%s\n",$$qvs);
    }
  }
}
