#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;

# Environmental variables:
#   MAKEDB_RREQ    options for qsub in creating databases (blast)
#   BLAST_RREQ     options for qsub in searching databases (blast)
#   BLAST_OPT      options for blast in searching databases
#   ASSEMBLY_RREQ  options for qsub in assembly (celera)

my $DEBUG=0;

my $valid_depth=4;
my $valid_read_length=500;

GetOptions(
  "debug"=>\$DEBUG
);

my %params;

if(@ARGV != 2){
  die "USAGE: <this> <ec.spec> <asm.spec>\n\t[-debug: outputs intermediate files]\n";
}

my $spec = $ARGV[1];

{
  my $ec_spec = $ARGV[0];
  open my $fh,"<",$ec_spec or die "cannot open $ec_spec :$!\n";
  while($_ = <$fh>){
    next if($_ =~ /^\s+$/);
    next if($_ =~ /^\s*#/);
    chomp;
    my @line = split /\s+/,$_;
  #  if(@line < 2){
  #    die "strange line in $ec_spec\n$_\n";
  #  }
    for(my $i=0; $i<@line; ++$i){
      if($line[$i] =~ /^\s*#/){
        @line = @line[0..$i-1];
        last;
      }
    }
    $params{$line[0]}=join(" ",@line[1..@line-1]);
    if($params{$line[0]}){
  #    printf("%s %s#\n",$line[0],$params{$line[0]});
    }
  }
  close $fh;
}

my $input_fastq;
my $from=0;
my $to=1;
my $partition=12;
my $word_size="";
my $evalue=1e-307;
#my $second_evalue=1e-307;
my $valid_voters=11;
my $trim=42;
my $estimated_genome_size=0;
my $ca_path="/home/imai/wgs-7.0/Linux-amd64/bin/";
my $blast_path="/home/imai/ppp/";
my $sprai_path="/usr/local/bin/";

my $preprefix="c";

my $outfmt = "7 qseqid sstart send sacc qstart qend bitscore evalue pident qseq sseq";

my $index=0;
my $command="";

if(defined($params{input_fastq})){
  $input_fastq = $params{input_fastq};
  if(!-e $input_fastq){
    die "$input_fastq does not exist.\n";
  }
}
else{
  die "specify input_fastq in ec.spec\n";
}

if(defined($params{estimated_genome_size})){
  $estimated_genome_size = $params{estimated_genome_size};
}
else{
  die "specify estimated_genome_size in ec.spec\n";
}
if($estimated_genome_size<=0){
  die "estimated_genome_size must be > 0\n";
}

if(defined($params{from})){
  $from = $params{from};
}
if(defined($params{to})){
  $to = $params{to};
}
if(defined($params{partition})){
  $partition = $params{partition};
}
if(defined($params{word_size})){
  $word_size = $params{word_size};
}
if(defined($params{evalue})){
  $evalue = $params{evalue};
}
#if(defined($params{second_evalue})){
#  $second_evalue = $params{second_evalue};
#}
if(defined($params{valid_voters})){
  $valid_voters = $params{valid_voters};
}
else{
  $valid_voters = 11;
#  my $n_base = -s $input_fastq;
#  $n_base /= 2;
#  $valid_voters = int(0.8*$n_base/$estimated_genome_size);
#  $valid_voters = ($valid_voters < 11) ? 11 : $valid_voters;
}
if(defined($params{trim})){
  $trim = $params{trim};
}
if(defined($params{ca_path})){
  $ca_path = $params{ca_path};
}
if(defined($params{blast_path})){
  $blast_path = $params{blast_path};
}
if(defined($params{sprai_path})){
  $sprai_path = $params{sprai_path};
}

printf STDERR ("-- params --\n");
printf STDERR ("input_fastq %s\n",$input_fastq);
printf STDERR ("estimated_genome_size %g\n",$estimated_genome_size);
printf STDERR ("from %s\n",$from);
printf STDERR ("to %s\n",$to);
printf STDERR ("partition %s\n",$partition);
printf STDERR ("word_size %d\n",$word_size);
printf STDERR ("evalue %g\n",$evalue);
#printf STDERR ("second_evalue %g\n",$second_evalue);
printf STDERR ("valid_voters %s\n",$valid_voters);
printf STDERR ("trim %d\n",$trim);
printf STDERR ("ca_path %s\n",$ca_path);
printf STDERR ("blast_path %s\n",$blast_path);
printf STDERR ("sprai_path %s\n",$sprai_path);
printf STDERR ("-- params --\n");

#my $rakuraku="./ezez4qsub_v4.pl";#TODO
my $log="./ezez4makefile.log";
#my $finish="finish_v3.pl";
#my $finish_log="./finish_v3.log";
#my $ikki="ca_ikki_v3.pl";
#my $ikki_log="./ca_ikki_v3.log";
#my $spec="./asm.spec";

#if($DEVEL){
#  $rakuraku = "./".$rakuraku;
##  $finish = "./".$finish;
##  $ikki = "./".$ikki;
#}

#`which $rakuraku 1>/dev/null 2>&1`;
#if($? != 0){
#  die "$rakuraku does not exist. $?\n";
#}

#`which $finish 1>/dev/null 2>&1`;
#if($? != 0){
#  die "$finish does not exist. $?\n";
#}

#`which $ikki 1>/dev/null 2>&1`;
#if($? != 0){
#  die "$ikki does not exist. $?\n";
#}

#my $ca_command_1="ln -s $ca_path";
#if(!-e "bin"){
#  `$ca_command_1`;
#}

#if(!-e $ca_path){
#  die "$ca_path does not exist.\n";
#}

#if(!-e $spec){
#  die "$spec does not exist.\n";
#}

if(!-e $input_fastq){
  die "$input_fastq does not exist.\n";
}

#my $ec_command="$rakuraku ";
#if($DEVEL){
#  $ec_command .= "-devel ";
#}
if(length($blast_path) > 0 && $blast_path !~ /\/$/){
  $blast_path .= "/";
}
if(length($sprai_path) > 0 && $sprai_path !~ /\/$/){
  $sprai_path .= "/";
}

#$ec_command .= "-input $input_fastq -from $from -to $to -partition $partition -evalue $evalue -second_evalue $second_evalue -valid_voters $valid_voters -trim $trim -blast_path $blast_path -sprai_path $sprai_path > $log 2>&1";
#my $finish_command="$finish ";
#if($DEVEL){
#  $finish_command .= "-devel ";
#}
#$finish_command .= "-from $from -to $to -partition $partition > $finish_log 2>&1";
#my $ca_command_2="$ikki ";
#if($DEVEL){
#  $ca_command_2 .= "-devel ";
#}
#$ca_command_2 .= "-from $from -to $to $spec $estimated_genome_size > $ikki_log 2>&1";
#if($opt_dryrun){
#  printf("%s\n",$ec_command);
##  printf("%s\n",$finish_command);
##  printf("%s\n",$ca_command_2);
#  exit;
#}
#`$ec_command`;
#`$finish_command`;
#`$ca_command_2`;

#if($from == 0 && !$input_fastq){
#  die "give an input fastq file\n";
#}

my $date="";
my $message="";

my $now = `date +%Y%m%d%H%M%S`;
chomp $now;

my $bindir=$sprai_path;
my $path2blast=$blast_path;
#if($DEVEL){
#  $bindir = `pwd`;
#  chomp $bindir;
##  $path2blast="/home/imai/port/bin/";
#}

if(!-d $bindir){
  die "$bindir does not exist\n"
}
else{
  #printf STDERR ("$bindir exists. ok.\n");
}
if(!-d $path2blast){
  die "$path2blast does not exist\n"
}
else{
  #printf STDERR ("$path2blast exists. ok.\n");
}

my $pwd = `pwd`;
chomp $pwd;

my $outdir = "$pwd/result";
if(!-d $outdir){
  mkdir "$outdir" or die "cannot mkdir $outdir: $!\n";
}

my @allscripts=();
my $script;
my @alloutputfiles=();
my @allerrfiles=();
my $outputfile;
my $errfile;

my @list=(0..$partition-1);

my $makefile = "$pwd/Makefile";
open my $mh, ">", $makefile or die "cannot open $makefile: $!\n";

my @current_dep=($input_fastq);
my @next_dep=();

if($from == 0)
{
  my $PG= "$bindir/fq2idfq.pl";

  my $iprefix = sprintf("$outdir/$preprefix%02d",0);
  $outputfile = sprintf("$iprefix.idfq.gz");
  my $command = sprintf("$PG \$\< --prefix $iprefix | gzip -c -1 > $outputfile.tmp && mv $outputfile.tmp $outputfile");

#  my $next_dependency = sprintf("$preprefix%02d_fq2idfq_$now",$from);
#  push(@next_dep,$next_dependency);
  push(@next_dep,$outputfile);

#  printf $mh ".PRECIOUS : $outputfile\n";
#  printf $mh "$next_dependency: ";
  printf $mh "$outputfile: ";
  for(my $i=0; $i<@current_dep; ++$i){
    printf $mh "$current_dep[$i] ";
  }
  printf $mh "\n";
  printf $mh "\t$command\n";
#  printf $mh "\ttouch \$\@\n";
  printf $mh "\n";
}

my $tmp_blastdb;
my $tmp_fa;
my $tmp_dfq;
my $tmp_idfq;

for(my $index=$from; $index<$to; ++$index){
  @current_dep = @next_dep;
  @next_dep = ();
  {
    my $rreq = "";
    if(exists $ENV{'MAKEDB_RREQ'}) {
      $rreq = "SGE_RREQ=\"$ENV{'MAKEDB_RREQ'}\" ";
    }
    $tmp_blastdb = sprintf("tmp_blastdb_%02d",$index);
    if(!-d "$outdir/$tmp_blastdb"){
      mkdir "$outdir/$tmp_blastdb" or die "cannot mkdir $outdir/$tmp_blastdb: $!\n";
    }
    my $PG = "$path2blast/makeblastdb";
    my $PG2= "$bindir/dfq2fq_v2.pl";
    $command = sprintf("${rreq}gzip -d -c $outdir/$preprefix%02d.idfq.gz | $PG2 -f - | $PG -in - -dbtype nucl -out $outdir/$tmp_blastdb/$preprefix%02d -title $preprefix%02d ",$index,$index,$index);

    my $next_dependency = sprintf("$outdir/$preprefix%02d_makeblastdb.done",$index);
    push(@next_dep,$next_dependency);

    my $targets = sprintf("$preprefix%02d.nhr $preprefix%02d.nin $preprefix%02d.nsq",$index,$index,$index);
    printf $mh "$next_dependency: ";
    for(my $i=0; $i<@current_dep; ++$i){
      printf $mh "$current_dep[$i] ";
    }
    printf $mh "\n";
    printf $mh "\t$command\n";
    printf $mh "\ttouch \$\@\n";
    printf $mh "\n";

  }

  {
    my $rreq = "";
    if(exists $ENV{'MAKEDB_RREQ'}) {
      $rreq = "SGE_RREQ=\"$ENV{'MAKEDB_RREQ'}\" ";
    }
    $tmp_fa = sprintf("tmp_fa_%02d",$index);
    if(!-d "$outdir/$tmp_fa"){
      mkdir "$outdir/$tmp_fa" or die "cannot mkdir $outdir/$tmp_fa: $!\n";
    }
    my $PG1= "$bindir/dfq2fq_v2.pl";
    my $PG2="$bindir/partition_fa.pl";
    $command = sprintf("${rreq}gzip -d -c $outdir/$preprefix%02d.idfq.gz | $PG1 -f - | $PG2 - %d -p $outdir/$tmp_fa/$preprefix%02d", $index,scalar(@list),$index);
    #$command = sprintf("gzip -d -c $outdir/$preprefix%02d.idfq.gz | $PG1 -f - | $PG2 - %d -z -p $outdir/$tmp_fa/$preprefix%02d", $index,scalar(@list),$index);

    my $next_dependency = sprintf("$outdir/$preprefix%02d_partition_fa.done",$index);
    push(@next_dep,$next_dependency);

    printf $mh ("$next_dependency : ");
    for(my $i=0; $i<@current_dep; ++$i){
      printf $mh "$current_dep[$i] ";
    }
    printf $mh "\n";
    printf $mh "\t$command\n";
    printf $mh "\ttouch \$\@\n";
    printf $mh "\n";
  }

  @current_dep=@next_dep;
  @next_dep=();

  {
    my $rreq = "";
    if(exists $ENV{'BLAST_RREQ'}) {
      $rreq = "SGE_RREQ=\"$ENV{'BLAST_RREQ'}\" ";
    }
    my $blast_ext_opt = "";
    if(exists $ENV{'BLAST_OPT'}) {
      $blast_ext_opt = $ENV{'BLAST_OPT'} . " ";
    }
    my $tmp_evalue;
    if($index >= 0){
      $tmp_evalue = $evalue;
    }
#    elsif($index > 0){
#      $tmp_evalue = $second_evalue;
#    }
    else{
      die "strange index: $index (index must be >= 0)\n";
    }
    $tmp_dfq = sprintf("tmp_dfq_%02d",$index);
    if(!-d "$outdir/$tmp_dfq"){
      mkdir "$outdir/$tmp_dfq" or die "cannot mkdir $outdir/$tmp_dfq: $!\n";
    }
    for(my $i=0; $i<@list; ++$i){
      $outputfile = sprintf("$outdir/$tmp_dfq/$preprefix%02d_%04d.dfq.gz",$index,$list[$i]);
      my $PG="$path2blast/blastn";
      if($word_size){
        $PG .= " -word_size $word_size";
      }
      my $PG2=sprintf("$bindir/bfmt72s -c $trim -u -i",$index);
      my $PG3="$bindir/nss2v -v $valid_voters";
      my $PG4="$bindir/myrealigner -f -B $valid_voters -b 3";
      #my $PG2=sprintf("$bindir/bfmt72s -d -c $trim -u -i -q $preprefix%02d.idfq.gz",$index);
      #my $PG3="$bindir/nss2v -e -v $valid_voters -q";
      #my $PG4="$bindir/myrealigner -f -q -B $valid_voters -b 3";
      # TODO
      if($DEBUG){
        my $debugdir = "$outdir/debug";
        if(!-d $debugdir){
          mkdir $debugdir or die "cannot mkdir $debugdir: $!\n";
        }
        my $myfmt7dir = sprintf("$debugdir/myfmt7_%02d",$index);
        if(!-d $myfmt7dir){
          mkdir $myfmt7dir or die "cannot mkdir $myfmt7dir: $!\n";
        }
        my $nssdir = sprintf("$debugdir/nss_%02d",$index);
        if(!-d $nssdir){
          mkdir $nssdir or die "cannot mkdir $nssdir: $!\n";
        }
        my $verticaldir = sprintf("$debugdir/vertical_%02d",$index);
        if(!-d $verticaldir){
          mkdir $verticaldir or die "cannot mkdir $verticaldir: $!\n";
        }
        my $realigneddir = sprintf("$debugdir/realigned_%02d",$index);
        if(!-d $realigneddir){
          mkdir $realigneddir or die "cannot mkdir $realigneddir: $!\n";
        }
        $PG4="$bindir/myrealigner -q -B $valid_voters -b 3";
        my $PG5="$bindir/myrealigner -f -n -q -B $valid_voters -b 3";
        $command = sprintf("${rreq}cat $outdir/$tmp_fa/$preprefix%02d_%04d.fa | $PG $blast_ext_opt -db $outdir/$tmp_blastdb/$preprefix%02d -query - -evalue $tmp_evalue -outfmt '$outfmt' | tee $myfmt7dir/$preprefix%02d_%04d.blastn | $PG2 - | tee $nssdir/$preprefix%02d_%04d.nss | $PG3 - | tee $verticaldir/$preprefix%02d_%04d.vertical | $PG4 - | tee $realigneddir/$preprefix%02d_%04d.realigned | $PG5 - | gzip -c -1 > $outputfile.tmp && mv $outputfile.tmp $outputfile", $index,$list[$i],$index,$index,$list[$i],$index,$list[$i],$index,$list[$i],$index,$list[$i]);
        #$command = sprintf("gzip -d -c $outdir/$tmp_fa/$preprefix%02d_%04d.fa.gz | $PG -db $outdir/$tmp_blastdb/$preprefix%02d -query - -evalue $tmp_evalue -outfmt '$outfmt' | tee $myfmt7dir/$preprefix%02d_%04d.blastn | $PG2 - | tee $nssdir/$preprefix%02d_%04d.nss | $PG3 - | tee $verticaldir/$preprefix%02d_%04d.vertical | $PG4 - | tee $realigneddir/$preprefix%02d_%04d.realigned | $PG5 - | gzip -c -1 > $outputfile", $index,$list[$i],$index,$index,$list[$i],$index,$list[$i],$index,$list[$i],$index,$list[$i]);
      }
      else{
        $command = sprintf("${rreq}cat $outdir/$tmp_fa/$preprefix%02d_%04d.fa | $PG $blast_ext_opt -db $outdir/$tmp_blastdb/$preprefix%02d -query - -evalue $tmp_evalue -outfmt '$outfmt' | $PG2 - | $PG3 - | $PG4 - | gzip -c -1 > $outputfile.tmp && mv $outputfile.tmp $outputfile", $index,$list[$i],$index,$index,$list[$i]);
        #$command = sprintf("gzip -d -c $outdir/$tmp_fa/$preprefix%02d_%04d.fa.gz | $PG -db $outdir/$tmp_blastdb/$preprefix%02d -query - -evalue $tmp_evalue -outfmt '$outfmt' | $PG2 - | $PG3 - | $PG4 - | gzip -c -1 > $outputfile.tmp && mv $outputfile.tmp $outputfile", $index,$list[$i],$index,$index,$list[$i]);
      }

#      my $next_dependency = sprintf("$preprefix%02d_dfq_%04d_$now",$index,$list[$i]);
#      push(@next_dep,$next_dependency);
      push(@next_dep,$outputfile);

#      printf $mh (".PRECIOUS: $outputfile\n");
#      printf $mh ("$next_dependency : ");
      printf $mh ("$outputfile : ");
      for(my $j=0; $j<@current_dep; ++$j){
        printf $mh ("$current_dep[$j] ");
      }
      printf $mh ("\n");
      printf $mh ("\t$command\n");
#      printf $mh ("\ttouch \$\@\n");
      printf $mh ("\n");
    }
  }

  @current_dep = @next_dep;
  @next_dep = ();

  {
    my $rreq = "";
    if(exists $ENV{'MAKEDB_RREQ'}) {
      $rreq = "SGE_RREQ=\"$ENV{'MAKEDB_RREQ'}\" ";
    }
    $tmp_idfq = sprintf("tmp_idfq_%02d",$index);
    if(!-d "$outdir/$tmp_idfq"){
      mkdir "$outdir/$tmp_idfq" or die "cannot mkdir $outdir/$tmp_idfq: $!\n";
    }
    for(my $i=0; $i<@list; ++$i){
      $outputfile = sprintf("$outdir/$tmp_idfq/$preprefix%02d.refined_%04d.idfq.gz",$index,$list[$i]);
      $command = sprintf("${rreq}gzip -d -c $outdir/$tmp_dfq/$preprefix%02d_%04d.dfq.gz | $bindir/dfq2fq_v2.pl - | gzip -c -1 > $outputfile.tmp && mv $outputfile.tmp $outputfile", $index,$list[$i]);

#      my $next_dependency = sprintf("$preprefix%02d_refined_%04d_$now",$index,$list[$i]);
#      push(@next_dep,$next_dependency);
      push(@next_dep,$outputfile);

#      printf $mh (".PRECIOUS: $outputfile\n");
#      printf $mh ("$next_dependency : ");
      printf $mh ("$outputfile : ");
      for(my $i=0; $i<@current_dep; ++$i){
        printf $mh ("$current_dep[$i] ");
      }
      printf $mh ("\n");
      printf $mh ("\t$command\n");
#      printf $mh ("\ttouch \$\@\n");
      printf $mh ("\n");
    }
  }

  @current_dep = @next_dep;
  @next_dep = ();

  {
    $outputfile = sprintf("$outdir/$preprefix%02d.idfq.gz",$index+1);
    my $files="";
    for(my $i=0; $i<@list; ++$i){
        $files .=  sprintf("$outdir/$tmp_idfq/$preprefix%02d.refined_%04d.idfq.gz ",$index,$list[$i]);
    }

    $command = sprintf("cat $files > $outputfile.tmp && mv $outputfile.tmp $outputfile");
#    my $next_dependency = sprintf("$preprefix%02d_cat_$now",$index);
#    push(@next_dep,$next_dependency);
    push(@next_dep,$outputfile);

#    printf $mh (".PRECIOUS: $outputfile\n");
#    printf $mh ("$next_dependency : ");
    printf $mh ("$outputfile : ");
    for(my $i=0; $i<@current_dep; ++$i){
      printf $mh ("$current_dep[$i] ");
    }
    printf $mh ("\n");
    printf $mh ("\t$command\n");
#    printf $mh ("\ttouch \$\@\n");
    printf $mh ("\n");
  }
}

#my $all = sprintf ("all: $next_dep[0]");

#close $mh;

#my $tmpfile = "$pwd/tmp.$now";
#`cp $makefile $tmpfile`;
#open $mh, ">", $makefile or die "cannot open $makefile: $!\n";
#printf $mh ("$all\n");
#printf $mh "\n";
#close $mh;

#`cat $tmpfile >> $makefile`;
#`rm $tmpfile`;

my $tmp_finish;

for(my $index=$from; $index<$to; ++$index){
  @current_dep = @next_dep;
  @next_dep = ();

  {
    my $rreq = "";
    if(exists $ENV{'MAKEDB_RREQ'}) {
      $rreq = "SGE_RREQ=\"$ENV{'MAKEDB_RREQ'}\" ";
    }
    $tmp_finish = sprintf("tmp_finish_%02d",$index);
    if(!-d "$outdir/$tmp_finish"){
      mkdir "$outdir/$tmp_finish" or die "cannot mkdir $outdir/$tmp_finish: $!\n";
    }
    $tmp_dfq = sprintf("tmp_dfq_%02d",$index);
    for(my $i=0; $i<$partition; ++$i){
      $outputfile = sprintf("$outdir/$tmp_finish/$preprefix%02d.refined_%04d.fin.idfq.gz",$index,$list[$i]);
      my $PG1 = "$bindir/dfq2fq_v2.pl";
      my $inputfile = sprintf("$outdir/$tmp_dfq/$preprefix%02d_%04d.dfq.gz",$index,$i);
      my $command = sprintf("${rreq}gzip -d -c $inputfile | $PG1 - -finish -valid_depth $valid_depth -valid_read_length $valid_read_length | gzip -c -1 > $outputfile.tmp && mv $outputfile.tmp $outputfile");
      #my $command = sprintf("gzip -d -c $outdir/$tmp_dfq/$preprefix%02d_%04d.dfq.gz | $PG1 - -finish -valid_depth $valid_depth -valid_read_length $valid_read_length | gzip -c -1 > $outputfile", $index,$i);

      push(@next_dep,$outputfile);

      printf $mh ("$outputfile : ");
      printf $mh ("$inputfile\n");
#      for(my $i=0; $i<@current_dep; ++$i){
#        printf $mh ("$current_dep[$i] ");
#      }
#      printf $mh ("\n");
      printf $mh ("\t$command\n");
      printf $mh ("\n");
    }
  }

  @current_dep = @next_dep;
  @next_dep = ();

  {
    my $outputfile = sprintf("$outdir/$preprefix%02d.fin.idfq.gz",$index+1);
    my $files="";
    for(my $i=0; $i<@list; ++$i){
        $files .=  sprintf("$outdir/$tmp_finish/$preprefix%02d.refined_%04d.fin.idfq.gz ",$index,$list[$i]);
    }

    $command = sprintf("cat $files > $outputfile.tmp && mv $outputfile.tmp $outputfile");
#    my $next_dependency = sprintf("$preprefix%02d_cat_$now",$index);
#    push(@next_dep,$next_dependency);
    push(@next_dep,$outputfile);

#    printf $mh (".PRECIOUS: $outputfile\n");
#    printf $mh ("$next_dependency : ");
    printf $mh ("$outputfile : ");
    for(my $i=0; $i<@current_dep; ++$i){
      printf $mh ("$current_dep[$i] ");
    }
    printf $mh ("\n");
    printf $mh ("\t$command\n");
#    printf $mh ("\ttouch \$\@\n");
    printf $mh ("\n");
  }
}

my $ecdone = "ec.done";
printf $mh ("$ecdone: $next_dep[0]\n");
printf $mh ("\ttouch \$\@;\n");
printf $mh ("\n");

my $ec_only = sprintf ("ec_only: $ecdone");
#my $ec_only = sprintf ("ec_only: $next_dep[0]");

#close $mh;

#$now = `date +%Y%m%d%H%M%S`;
#my $tmpfile = "$pwd/tmp.$now";
#`cp $makefile $tmpfile`;
#open $mh, ">", $makefile or die "cannot open $makefile: $!\n";
#printf $mh ("$ec_only\n");
#printf $mh "\n";
#close $mh and `cat $tmpfile >> $makefile`;
#`rm $tmpfile`;

# assembly

#open $mh, ">>", $makefile or die "cannot open $makefile: $!\n";

my $bashcommand="";

my $suffix = "top20x";

@current_dep = ($ecdone);
@next_dep = ();

for(my $i=$from+1; $i<=$to; ++$i){
  my $rreq = "";
  if(exists $ENV{'MAKEDB_RREQ'}) {
    $rreq = "SGE_RREQ=\"$ENV{'MAKEDB_RREQ'}\" ";
  }
  my $prefix=sprintf("c%02d.fin",$i);
  my $PG1 = "$bindir/get_top_20x_fa.pl";
  my $out = "$outdir/$prefix.$suffix.fq";
  push(@next_dep,$out);
  printf $mh ("$out : $outdir/$prefix.idfq.gz\n");
#  printf $mh ("$prefix.$suffix.fq : ");
#  for(my $j=0; $j<@current_dep; ++$j){
#    printf $mh ("$current_dep[$j] ");
#  }
#  printf $mh ("\n");
  printf $mh ("\t${rreq}gzip -d -c \$^ | $PG1 - -g $estimated_genome_size -q > \$\@.tmp && mv \$\@.tmp \$\@\n");
  printf $mh ("\n");
#  $bashcommand .= "(gzip -d -c $pwd/$prefix.idfq.gz | $PG1 - -g $estimated_genome_size -q > $prefix.$suffix.fq) &\n";
}

#`
#  $bashcommand
#  wait
#`;
#printf STDERR ("done idfq2fq\n");

@current_dep = @next_dep;
@next_dep = ();

#printf STDERR ("start fastqToCA\n");
for(my $i=$from+1; $i<=$to; ++$i){
  my $prefix=sprintf("c%02d.fin",$i);
  my $out = sprintf("$outdir/$prefix.$suffix.frg");
  push(@next_dep,$out);
  printf $mh ("$out : ");
  for(my $j=0; $j<@current_dep; ++$j){
    printf $mh ("$current_dep[$j] ");
  }
  printf $mh ("\n");
  printf $mh ("\t$ca_path/fastqToCA -libraryname foo -technology pacbio -reads $outdir/$prefix.$suffix.fq > \$\@\n");
  printf $mh ("\n");
#  `bin/fastqToCA -libraryname foo -technology pacbio -reads $prefix.$suffix.fq > $prefix.$suffix.frg`;
}
#printf STDERR ("done fastqToCA\n");

#printf STDERR ("start CA (stopAfter=unitigger)\n");

$now = `date +%Y%m%d%H%M%S`;
chomp $now;
my $now_used=0;

@current_dep = @next_dep;
@next_dep = ();

for(my $i=$from+1; $i<=$to; ++$i){
  my $rreq = "";
  if(exists $ENV{'ASSEMBLY_RREQ'}) {
    $rreq = "SGE_RREQ=\"$ENV{'ASSEMBLY_RREQ'}\" ";
  }
  my $prefix=sprintf("c%02d.fin",$i);
  if(-e "$prefix.$suffix"){
    $now_used=1;
    my $out = sprintf("$outdir/ca_unitigger_${now}_%02d.done",$i);
    push(@next_dep, $out);
    printf $mh ("$out : ");
    for(my $j=0; $j<@current_dep; ++$j){
      printf $mh ("$current_dep[$j] ");
    }
    printf $mh ("\n");
    printf $mh ("\t${rreq}$ca_path/runCA stopAfter=unitigger -dir $outdir/$prefix.${suffix}_$now -p $prefix.${suffix}_$now -s $spec $outdir/$prefix.$suffix.frg |& tee -a $outdir/do_${now}_$prefix.$suffix.log\n");
    printf $mh ("\ttouch $out\n");
    printf $mh ("\n");

#    `bin/runCA stopAfter=unitigger -dir ${now}_$prefix.$suffix -p ${now}_$prefix.$suffix -s $spec $prefix.$suffix.frg |& tee -a do_${now}_$prefix.$suffix.log`;
  }
  else{
    my $out = sprintf("$outdir/ca_unitigger_%02d.done",$i);
    push(@next_dep, $out);
    printf $mh ("$out : ");
    for(my $j=0; $j<@current_dep; ++$j){
      printf $mh ("$current_dep[$j] ");
    }
    printf $mh ("\n");
    printf $mh ("\t${rreq}$ca_path/runCA stopAfter=unitigger -dir $outdir/$prefix.$suffix -p $prefix.$suffix -s $spec $outdir/$prefix.$suffix.frg |& tee -a $outdir/do_$prefix.$suffix.log\n");
    printf $mh ("\ttouch $out\n");
    printf $mh ("\n");
#    `bin/runCA stopAfter=unitigger -dir $prefix.$suffix -p $prefix.$suffix -s $spec $prefix.$suffix.frg |& tee -a do_$prefix.$suffix.log`;
  }
}

@current_dep = @next_dep;
@next_dep = ();

for(my $i=$from+1; $i<=$to; ++$i){
  my $rreq = "";
  if(exists $ENV{'ASSEMBLY_RREQ'}) {
    $rreq = "SGE_RREQ=\"$ENV{'ASSEMBLY_RREQ'}\" ";
  }
  my $prefix=sprintf("c%02d.fin",$i);
  if($now_used==1){
    my $out = sprintf("$outdir/ca_all_${now}_%02d.done",$i);
    push(@next_dep, $out);
    printf $mh ("$out : ");
    for(my $j=0; $j<@current_dep; ++$j){
      printf $mh ("$current_dep[$j] ");
    }
    printf $mh ("\n");
    printf $mh ("\t${rreq}$ca_path/runCA -dir $outdir/$prefix.${suffix}_$now -p $prefix.${suffix}_$now -s $spec $outdir/$prefix.$suffix.frg |& tee -a $outdir/do_${now}_$prefix.$suffix.log\n");
    printf $mh ("\ttouch $out\n");
    printf $mh ("\n");

#    $bashcommand .= "(bin/runCA -dir ${now}_$prefix.$suffix -p ${now}_$prefix.$suffix -s $spec $prefix.$suffix.frg |& tee -a do_${now}_$prefix.$suffix.log) &\n";
  }
  else{
    my $out = sprintf("$outdir/ca_all_%02d.done",$i);
    push(@next_dep, $out);
    printf $mh ("$out : ");
    for(my $j=0; $j<@current_dep; ++$j){
      printf $mh ("$current_dep[$j] ");
    }
    printf $mh ("\n");
    printf $mh ("\t${rreq}$ca_path/runCA -dir $outdir/$prefix.$suffix -p $prefix.$suffix -s $spec $outdir/$prefix.$suffix.frg |& tee -a $outdir/do_$prefix.$suffix.log\n");
    printf $mh ("\ttouch $out\n");
    printf $mh ("\n");
#    $bashcommand .= "(bin/runCA -dir $prefix.$suffix -p $prefix.$suffix -s $spec $prefix.$suffix.frg |& tee -a do_$prefix.$suffix.log) &\n";
  }
}

@current_dep = @next_dep;
@next_dep = ();

close $mh;
$now = `date +%Y%m%d%H%M%S`;
chomp $now;
my $tmpfile = "$pwd/tmp.$now";
`cp $makefile $tmpfile`;

open $mh, ">", $makefile or die "cannot open $makefile: $!\n";
printf $mh ("all: ");
for(my $i=0; $i<@current_dep; ++$i){
  printf $mh ("$current_dep[$i] ");
}
printf $mh ("\n");
printf $mh ("\n");

printf $mh $ec_only;
printf $mh ("\n");
printf $mh ("\n");

close $mh;

`cat $tmpfile >> $makefile`;
`rm $tmpfile`;


