#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;

my $DEBUG=0;
my $DEVEL=0;

my $CURRENT=12;
my $valid_depth=4;
my $valid_read_length=500;
my $scriptdir = "sgescript";
my $data_dir_prefix="data";
my $preprefix="c";
my $ppp="p";

my $confident_depth = $valid_depth;
my $confident_length_coefficient = 0.75;

my $opt_dryrun;
my $opt_finish;
my $opt_ec_only;

my $now = `date +%Y%m%d_%H%M%S`;
chomp $now;

GetOptions("n" => \$opt_dryrun,"devel" => \$DEVEL,"debug"=>\$DEBUG,"finish"=>\$opt_finish,"now=s"=>\$now,"ec_only"=>\$opt_ec_only);

if($opt_finish){
  $opt_dryrun=1;
}

if($DEBUG){
  die "debug mode is not supported yet, sorry.\n";
}

my %params;

if(@ARGV != 2){
  die "USAGE: <this> <ec.spec> <asm.spec>\n\t[-debug: outputs intermediate files (not implemented)]\n\t[-n: outputs qsub scripts and does NOT qsub]\n\t[-now string: a suffix of some directories]\n\t[-ec_only: not assemble]\n";
  #die "USAGE: <this> estimated_genome_size(integer>0) [-i integer (iterations)]\n";
}

my $pwd = `pwd`;
chomp $pwd;

my $asm_spec = $ARGV[1];
$asm_spec =~ s/^\s+//;
if(!-e $asm_spec){
  die "$asm_spec does not exist.\n";
}

if($asm_spec =~ /^\//){
  # real path; do nothing
}
else{
  $asm_spec = "$pwd/$asm_spec";
}
#print "$asm_spec\n";
#exit;

{
  my $ec_spec = $ARGV[0];
  open my $fh,"<",$ec_spec or die "cannot open $ec_spec :$!\n";
  while($_ = <$fh>){
    next if($_ =~ /^\s+$/);
    next if($_ =~ /^\s*#/);
    chomp;
    my @line = split /\s+/,$_;
  #  if(@line < 2){
  #    die "strange line in $ec_spec\n$_\n";
  #  }
    for(my $i=0; $i<@line; ++$i){
      if($line[$i] =~ /^\s*#/){
        @line = @line[0..$i-1];
        last;
      }
    }
    $params{$line[0]}=join(" ",@line[1..@line-1]);
    if($params{$line[0]}){
  #    printf("%s %s#\n",$line[0],$params{$line[0]});
    }
  }
  close $fh;
}

#exit;

my $input_fastq="";
my $from=0;
my $to=1;
my $pre_partition=1;
my $partition="";
my $word_size=11;
my $evalue=1e-50;
#my $second_evalue="";
my $valid_voters="";
my $trim="";
my $estimated_genome_size="";
my $ca_path="";
my $blast_path="";
my $sprai_path="";
my $queue_req="";
my $blast_rreq="";
my $rerun=-1;
my $blasr_path="";
my $blasr_opt="";

if(defined($params{input_fastq})){
  $input_fastq = $params{input_fastq};
  if(!-e $input_fastq){
    die "$input_fastq does not exist.\n";
  }
}
else{
  die "specify input_fastq in ec.spec\n";
}

if(defined($params{estimated_genome_size})){
  $estimated_genome_size = $params{estimated_genome_size};
}
else{
  die "specify estimated_genome_size in ec.spec\n";
}
if($estimated_genome_size<=0){
  die "estimated_genome_size must be > 0\n";
}

if(defined($params{from})){
  $from = $params{from};
}
if(defined($params{to})){
  $to = $params{to};
}
if(defined($params{pre_partition})){
  $pre_partition = $params{pre_partition};
}
if(defined($params{partition})){
  $partition = $params{partition};
}
if(defined($params{word_size})){
  $word_size = $params{word_size};
}
if(defined($params{evalue})){
  $evalue = $params{evalue};
}
#if(defined($params{second_evalue})){
#  $second_evalue = $params{second_evalue};
#}
if(defined($params{valid_voters})){
  $valid_voters = $params{valid_voters};
}
else{
#  my $fq2fa="fq2fa.pl";
#  if($DEVEL){
#    $fq2fa="./".$fq2fa;
#  }
#  my $n_base=`$fq2fa $input_fastq | grep -v \"^>\" | tr -d \"\\n\" | wc -m`;
#  chomp $n_base;

#  $valid_voters = 11;

  my $n_base = -s $input_fastq;
  $n_base /= 2;
  $valid_voters = int(0.8*$n_base/$estimated_genome_size);
  $valid_voters = ($valid_voters < 11) ? 11 : $valid_voters;
}
if(defined($params{trim})){
  $trim = $params{trim};
}
if(defined($params{ca_path})){
  $ca_path = $params{ca_path};
}
if(defined($params{blast_path})){
  $blast_path = $params{blast_path};
}
if(defined($params{sprai_path})){
  $sprai_path = $params{sprai_path};
}
if(defined($params{queue_req})){
  $queue_req = $params{queue_req};
}
if(defined($params{BLAST_RREQ})){
  $blast_rreq = $params{BLAST_RREQ};
}
if(defined($params{rerun})){
  $rerun = $params{rerun};
}
if(defined($params{blasr_path})){
  $blasr_path = $params{blasr_path};
}
if(defined($params{blasr_opt})){
  $blasr_opt = $params{blasr_opt};
}

printf STDERR ("#>- params -<#\n");
printf STDERR ("input_fastq %s\n",$input_fastq);
printf STDERR ("estimated_genome_size %g\n",$estimated_genome_size);
printf STDERR ("from %s\n",$from);
printf STDERR ("to %s\n",$to);
printf STDERR ("pre_partition %s\n",$pre_partition);
printf STDERR ("partition %s\n",$partition);
if($word_size){
  printf STDERR ("word_size %d\n",$word_size);
}
printf STDERR ("evalue %g\n",$evalue);
#printf STDERR ("second_evalue %g\n",$second_evalue);
printf STDERR ("valid_voters %s\n",$valid_voters);
printf STDERR ("trim %d\n",$trim);
printf STDERR ("ca_path %s\n",$ca_path);
printf STDERR ("blast_path %s\n",$blast_path);
printf STDERR ("sprai_path %s\n",$sprai_path);
printf STDERR ("queue_req %s\n",$queue_req);
printf STDERR ("blast_rreq %s\n",$blast_rreq);
if($rerun >= 0){
  printf STDERR ("rerun %s\n",$rerun);
}
if($DEVEL){
  printf STDERR ("development_mode\t%s\n","true");
}
if($DEBUG){
  printf STDERR ("debug_mode\t%s\n","true");
}
if($blasr_path){
  printf STDERR ("blasr_path\t%s\n",$blasr_path);
  if($blasr_opt){
    printf STDERR ("blasr_opt\t%s\n",$blasr_opt);
  }
}
printf STDERR ("#>- params -<#\n");

if(!-e $input_fastq){
  die "$input_fastq does not exist.\n";
}

if(length($blast_path) > 0 && $blast_path !~ /\/$/){
  $blast_path .= "/";
}
if(length($sprai_path) > 0 && $sprai_path !~ /\/$/){
  $sprai_path .= "/";
}

my $outfmt = "7 qseqid sstart send sacc qstart qend bitscore evalue pident qseq sseq";
#my $maxfilesize="100GB";

my $command="";



if($from == 0 && !-f "$pwd/$input_fastq"){
  die "give an input fastq file\n";
}

my $date="";
my $message="";





#my $now = `date +%Y%m%d%H%M%S`;
#chomp $now;

my $bindir=$sprai_path;
my $path2blast=$blast_path;

if($DEVEL){
  $bindir = `pwd`;
  chomp $bindir;
#  $path2blast="/home/imai/port/bin/";
}

if(!-d $bindir){
  die "$bindir does not exist\n"
}
else{
  #printf STDERR ("$bindir exists. ok.\n");
}
if(!-d $path2blast && !-d $blasr_path){
  die "specify aligner dir. $path2blast or $blasr_path does not exist\n"
}
else{
  #printf STDERR ("$path2blast exists. ok.\n");
}

#my $now2;
#{
#  $now2 = `date +%Y%m%d_%H%M%S`;
#  chomp $now2;
#  $scriptdir = "$pwd/${scriptdir}_$now2";
#}
$scriptdir = "$pwd/${scriptdir}_$now";
#`touch $scriptdir`;
#`rm -r $scriptdir`;
if(!-d $scriptdir){
  `mkdir $scriptdir`;
}

#my @allscripts=();
my @pre_array_jobs;
my @array_jobs;
my @post_array_jobs;

my $script;
#my @alloutputfiles=();
#my @allerrfiles=();
my $outputfile;
my $errfile;

my @list=(0..$partition-1);

#my $datadir="$pwd/${data_dir_prefix}_$now2";
my $datadir="$pwd/${data_dir_prefix}_$now";
if(!-d $datadir){
  mkdir $datadir or die "cannot mkdir $datadir: $!\n";
}

my $iprefix = sprintf("$preprefix%02d",0);
my $orig_idfq = "$datadir/$iprefix.idfq.gz";

if($from == 0 && $rerun < 0 && !$opt_finish)
{
  # fq -> idfq (& id2n)
  my $PG = "$bindir/fq2idfq.pl";
#  my $iprefix = sprintf("$preprefix%02d",0);
  my $not_passed = sprintf("$datadir/$preprefix%02d.not_passed.idfq.gz",0);
  my $command = "$PG $pwd/$input_fastq --prefix $datadir/$iprefix | gzip -c -1 > $orig_idfq && ln -sf $orig_idfq $not_passed";
  $script = sprintf("$scriptdir/fq2idfq.sh");
#  push(@allscripts,$script);
  open my $fh, ">", $script or die $!;
  printf $fh ("#!/bin/bash\n");
  printf $fh ("#\$ -S /bin/bash\n");
  printf $fh ("#\$ -cwd\n");
  printf $fh ("#\$ -N $preprefix%02d_fq2idfq_$now\n",0);
  printf $fh ("#\$ -o $datadir\n");
  printf $fh ("#\$ -e $datadir\n");
  printf $fh ("$command\n");
  close $fh;
  if($queue_req){
    `qsub $queue_req $script`;
  }
  else{
    `qsub $script`;
  }
}

if($rerun >= 0){
  $from = $rerun;
}

if($pre_partition and $pre_partition < 1){
  printf STDERR ("WARNING: given pre_partition %d was changed to 1\n",$pre_partition);
  $pre_partition = 1;
}

for(my $index=$from; $index<$to; ++$index){
  @pre_array_jobs=();
  @array_jobs=();
  @post_array_jobs=();

#  my $blastdbdir = sprintf("$datadir/$preprefix%02d",$index);
#  if(!-d $blastdbdir){
#    mkdir $blastdbdir or die "cannot mkdir $blastdbdir: $!\n";
#  }
  my $prepadir = sprintf("$datadir/$preprefix%02d",$index);
  if(!-d $prepadir){
    mkdir $prepadir or die "cannot mkdir $prepadir: $!\n";
  }
  my $script_sub_dir = sprintf("$scriptdir/$preprefix%02d",$index);
  if(!-d $script_sub_dir){
    mkdir $script_sub_dir or die "cannot mkdir $script_sub_dir: $!\n";
  }
  my $errdir = sprintf("$datadir/$preprefix%02d/err/",$index);
  my $doutdir = sprintf("$datadir/$preprefix%02d/dout/",$index);
  if(!-d $errdir){
    mkdir $errdir or die "cannot mkdir $errdir: $!\n";
  }
  if(!-d $doutdir){
    mkdir $doutdir or die "cannot mkdir $doutdir: $!\n";
  }

  if($word_size){
    if($word_size == 4){
      $to = $index;
      last;
    }
    my $foo = 1.0;
    for(my $i=0; $i < ($index-$from); ++$i){
      $foo = $foo*1.0/2.0;
    }
    $word_size = int(($word_size+0.0)*$foo);
    if($word_size < 4){
      $word_size = 4;
    }
  }

  my $passed_all="";
  for(my $i=($from>1)?$from:1; $i<=$index; ++$i){
    my $passed = sprintf("$datadir/$preprefix%02d.passed.fin.idfq.gz",$i);
    $passed_all .= $passed." ";
  }
  my $not_passed = sprintf("$datadir/$preprefix%02d.not_passed.idfq.gz",$index);

  if(!$opt_finish and ($rerun<0 or $index > $rerun))
  {
    my $PG;
    my $PG2;
    if($blasr_path){
      $PG2= "$bindir/dfq2fq_v2.pl";
      $command = sprintf("cat $passed_all $not_passed | gzip -d -c - | $PG2 -f - > $datadir/$preprefix%02d.fasta ",$index);
      $command .= sprintf(" && $blasr_path/sawriter $datadir/$preprefix%02d.fasta ",$index);
    }
    else{
      $PG = "$path2blast/makeblastdb";
      $PG2= "$bindir/dfq2fq_v2.pl";
      $command = sprintf("cat $passed_all $not_passed | gzip -d -c - | $PG2 -f - | $PG -in - -dbtype nucl -out $datadir/$preprefix%02d -title $preprefix%02d ",$index,$index);
    }
    if($index > 0){
      # do NOT make db.
      $command = "sleep 1";
    }

    $script = sprintf("$script_sub_dir/$preprefix%02d_makeblastdb.sh",$index);
    push(@pre_array_jobs,$script);
#    push(@allscripts,$script);
    open my $fh, ">", $script or die $!;
    printf $fh ("#!/bin/bash\n");
    printf $fh ("#\$ -S /bin/bash\n");
    printf $fh ("#\$ -cwd\n");
    printf $fh ("#\$ -N $preprefix%02d_makeblastdb_$now\n",$index);
    if($queue_req){
      printf $fh ("#\$ $queue_req\n");
    }
    printf $fh ("#\$ -o $datadir\n");
    printf $fh ("#\$ -e $datadir\n");
    if($index > $from){
      printf $fh ("#\$ -hold_jid $preprefix%02d_second_cat_not_passed_$now\n",$index-1);
      printf $fh ("#\$ -hold_jid $preprefix%02d_second_cat_passed_finish_$now\n",$index-1);
      #printf $fh ("#\$ -hold_jid $preprefix%02d_second_cat_pnp_finish_$now\n",$index-1);
    }
    elsif($from == 0){
      printf $fh ("#\$ -hold_jid $preprefix%02d_fq2idfq_$now\n",0);
    }
    printf $fh ("$command\n");
    close $fh;
  }

  # prepartitioning
  if(!$opt_finish and ($rerun<0 or $index > $rerun))
  {
    my $PG1= "$bindir/dfq2fq_v2.pl";
    my $PG2="$bindir/partition_fa.pl -1origin";
    $command = sprintf("cat $not_passed | gzip -d -c - | $PG1 -f - | $PG2 - %d -p $prepadir/$preprefix%02d", $pre_partition,$index);
    $script = sprintf("$script_sub_dir/$preprefix%02d_pre_partition_fa.sh",$index);
    push(@pre_array_jobs,$script);
#    push(@allscripts,$script);
    open my $fh, ">", $script or die $!;
    printf $fh ("#!/bin/bash\n");
    printf $fh ("#\$ -S /bin/bash\n");
    printf $fh ("#\$ -cwd\n");
    printf $fh ("#\$ -N $preprefix%02d_pre_partition_fa_$now\n",$index);
    if($queue_req){
      printf $fh ("#\$ $queue_req\n");
    }
    printf $fh ("#\$ -o $prepadir\n");
    printf $fh ("#\$ -e $prepadir\n");
    if($index > $from){
      printf $fh ("#\$ -hold_jid $preprefix%02d_second_cat_$now\n",$index-1);
    }
    elsif($from == 0){
      printf $fh ("#\$ -hold_jid $preprefix%02d_fq2idfq_$now\n",0);
    }
    printf $fh ("$command\n");
    close $fh;
  }


  my @second_cat_holdjids = ();
  my @second_cat_passed_finish_holdjids = ();
  my @second_cat_pnp_finish_holdjids = ();

  for(my $pp = 1,my $orig_datadir = $datadir; $pp<=$pre_partition; ++$pp,$datadir = $orig_datadir){
    my $ppdir = sprintf("$datadir/$preprefix%02d/$pp/",$index);
    if(!-d $ppdir){
      mkdir $ppdir or die "cannot mkdir $ppdir: $!\n";
    }
    my $maindir= $ppdir;
    if(!-d $maindir){
      mkdir $maindir or die "cannot mkdir $maindir: $!\n";
    }

    # sub partitioning
    if(!$opt_finish and ($rerun<0 or $index > $rerun))
    {
      my $PG="$bindir/partition_fa.pl";
      $command = sprintf("cat $prepadir/$preprefix%02d_%04d.fa | $PG - %d -p $maindir/$preprefix%02d", $index,$pp,scalar(@list),$index);
      $script = sprintf("$script_sub_dir/$preprefix%02d$ppp%02d_partition_fa.sh",$index,$pp);
      push(@pre_array_jobs,$script);
#      push(@allscripts,$script);
      open my $fh, ">", $script or die $!;
      printf $fh ("#!/bin/bash\n");
      printf $fh ("#\$ -S /bin/bash\n");
      printf $fh ("#\$ -cwd\n");
      printf $fh ("#\$ -N $preprefix%02d$ppp%02d_partition_fa_$now\n",$index,$pp);
      if($queue_req){
        printf $fh ("#\$ $queue_req\n");
      }
      printf $fh ("#\$ -o $doutdir\n");
      printf $fh ("#\$ -e $errdir\n");
      printf $fh ("#\$ -hold_jid $preprefix%02d_pre_partition_fa_$now\n",$index);
      printf $fh ("$command\n");
      close $fh;
    }
  }
#  print STDERR "pre_array_jobs printed\n";
  
  # array jobs
  my $ppdir = sprintf("$datadir/$preprefix%02d/\${SGE_TASK_ID}/",$index);
  my $maindir= $ppdir;
    if(!$opt_finish)
    {
      my $tmp_evalue;
      if($index >= 0){
        $tmp_evalue = $evalue;
      }
#      elsif($index > 0){
#        $tmp_evalue = $second_evalue;
#      }
      else{
        die "strange index: $index (index must be >= 0)\n";
      }
      for(my $i=0; $i<@list; ++$i){
        $outputfile = sprintf("$preprefix%02d_%04d.dfq.gz",$index,$list[$i]);
        my $PG="$path2blast/blastn";
        if($word_size){
          $PG .= " -word_size $word_size";
        }
        my $PG2=sprintf("$bindir/bfmt72s -c $trim -u -i",$index);
        my $PG3="$bindir/nss2v -v $valid_voters";
        my $PG4="$bindir/myrealigner -f -B $valid_voters -b 3";
        if($DEBUG){# TODO
          die "sorry. debug mode is under construction.\n";
          $PG4="$bindir/myrealigner -q -B $valid_voters -b 3";
          my $PG5="$bindir/myrealigner -f -n -q -B $valid_voters -b 3";
          $command = sprintf("cat $maindir/$preprefix%02d_%04d.fa | $PG -db $maindir/$preprefix%02d -query - -evalue $tmp_evalue -outfmt '$outfmt' | tee $datadir/$preprefix%02d_%04d.blastn | $PG2 - | tee $datadir/$preprefix%02d_%04d.nss | $PG3 - | tee $datadir/$preprefix%02d_%04d.vertical | $PG4 - | tee $datadir/$preprefix%02d_%04d.realigned | $PG5 - | gzip -c -1 > $maindir/$outputfile", $index,$list[$i],$index,$index,$list[$i],$index,$list[$i],$index,$list[$i],$index,$list[$i]);
        }
        else{
          if($blasr_path){
            my $PG6 = "$bindir/m52bfmt7";
            $command = sprintf("$blasr_path/blasr $maindir/$preprefix%02d_%04d.fa $datadir/$preprefix%02d.fasta -sa $datadir/$preprefix%02d.fasta.sa -m 5 $blasr_opt -out $maindir/tmp%02d_%04d.m5 ",$index,$list[$i],0,0,$index,$list[$i]);
            $command .= sprintf(" && cat $maindir/tmp%02d_%04d.m5 | $PG6 - |  $PG2 - | $PG3 - | $PG4 - | gzip -c -1 > $maindir/$outputfile", $index,$list[$i]);
            #$command = sprintf("$blasr_path/blasr $maindir/$preprefix%02d_%04d.fa $datadir/$preprefix%02d.fasta -sa $datadir/$preprefix%02d.fasta.sa -m 5 $blasr_opt -out $maindir/tmp%02d_%04d.m5 ",$index,$list[$i],$index,$index,$index,$list[$i]);
            #$command .= sprintf(" && cat $maindir/tmp%02d_%04d.m5 | $PG6 - |  $PG2 - | $PG3 - | $PG4 - | gzip -c -1 > $maindir/$outputfile", $index,$list[$i]);
          }
          else{
            $command = sprintf("cat $maindir/$preprefix%02d_%04d.fa | $PG -db $datadir/$preprefix%02d -query - -evalue $tmp_evalue -outfmt '$outfmt' | $PG2 - | $PG3 - | $PG4 - | gzip -c -1 > $maindir/$outputfile", $index,$list[$i],0);
            #$command = sprintf("cat $maindir/$preprefix%02d_%04d.fa | $PG -db $datadir/$preprefix%02d -query - -evalue $tmp_evalue -outfmt '$outfmt' | $PG2 - | $PG3 - | $PG4 - | gzip -c -1 > $maindir/$outputfile", $index,$list[$i],$index,$index,$list[$i]);
          }
        }
  
        $script = sprintf("$script_sub_dir/$preprefix%02d_ec_%04d.sh",$index,$list[$i]);
        push(@array_jobs,$script);
#        push(@allscripts,$script);
        open my $fh, ">", $script or die $!;
  
        printf $fh ("#!/bin/bash\n");
        printf $fh ("#\$ -S /bin/bash\n");
        printf $fh ("#\$ -cwd\n");
        printf $fh ("#\$ -N $preprefix%02d_ec_%04d_$now\n",$index,$list[$i]);
#        printf $fh ("#\$ -N $preprefix%02d$ppp%02d_ec_%04d_$now\n",$index,$pp,$list[$i]);
        if($queue_req){
          printf $fh ("#\$ $queue_req\n");
        }
        if($blast_rreq){
          printf $fh ("#\$ $blast_rreq\n");
        }
        printf $fh ("#\$ -o $doutdir\n");
        printf $fh ("#\$ -e $errdir\n");

        my $holdlist=sprintf("$preprefix%02d_makeblastdb_$now",$index);
        for(my $pp=1; $pp<=$pre_partition; ++$pp){
          $holdlist.=sprintf(",$preprefix%02d$ppp%02d_partition_fa_$now",$index,$pp);
        }
#        my $holdlist=sprintf("$preprefix%02d_makeblastdb_$now,$preprefix%02d$ppp%02d_partition_fa_$now",$index,$index,$pp);
        if($rerun<0 or $index > $rerun){
          printf $fh ("#\$ -hold_jid $holdlist\n");
        }
        printf $fh ("$command\n");
  
        close $fh;
      }
    }

    # not passed list
    if(!$opt_finish)
    {
      {
        my @tmp;
        for(my $i=0; $i<@list; ++$i){
          $tmp[$i]=sprintf("$preprefix%02d_ec_%04d_$now",$index,$list[$i]);
#          $tmp[$i]=sprintf("$preprefix%02d$ppp%02d_ec_%04d_$now",$index,$pp,$list[$i]);
        }
        my $holdlist=join(',',@tmp);
  
        for(my $i=0; $i<@list; $i+=$CURRENT){
          $outputfile = sprintf("$preprefix%02d.not_passed_%04d.list",$index,$list[$i]);
          $command = sprintf("gzip -d -c $maindir/$preprefix%02d_%04d.dfq.gz | $bindir/dfq2fq_v2.pl - --nlist --clc $confident_length_coefficient --cd $confident_depth > $maindir/$outputfile", $index,$list[$i]);
  
          $script = sprintf("$script_sub_dir/$preprefix%02d_not_passed_%04d.sh",$index,$list[$i]);
          push(@array_jobs,$script);
#          push(@allscripts,$script);
          open my $fh, ">", $script or die $!;
  
          printf $fh ("#!/bin/bash\n");
          printf $fh ("#\$ -S /bin/bash\n");
          printf $fh ("#\$ -cwd\n");
          printf $fh ("#\$ -N $preprefix%02d_not_passed_%04d_$now\n",$index,$list[$i]);
#          printf $fh ("#\$ -N $preprefix%02d$ppp%02d_dfq2fq_%04d_$now\n",$index,$pp,$list[$i]);
          if($queue_req){
            printf $fh ("#\$ $queue_req\n");
          }
          printf $fh ("#\$ -o $doutdir\n");
          printf $fh ("#\$ -e $errdir\n");
          printf $fh ("#\$ -hold_jid $holdlist\n");
          printf $fh ("$command\n");
          for(my $j=1; $j<$CURRENT && $i+$j<@list; ++$j){
            $command = sprintf("gzip -d -c $maindir/$preprefix%02d_%04d.dfq.gz | $bindir/dfq2fq_v2.pl - --nlist --clc $confident_length_coefficient --cd $confident_depth >> $maindir/$outputfile", $index,$list[$i+$j]);
            #$command = sprintf("gzip -d -c $maindir/$preprefix%02d_%04d.dfq.gz | $bindir/dfq2fq.pl - | gzip -c -1 >> $maindir/$outputfile", $index,$list[$i+$j]);
            printf $fh ("$command\n");
          }
  
          close $fh;
        }
      }
  
      {
        my @tmp;
        for(my $i=0; $i<@list; $i+=$CURRENT){
          $tmp[$i/$CURRENT]=sprintf("$preprefix%02d_not_passed_%04d_$now",$index,$list[$i]);
#          $tmp[$i/$CURRENT]=sprintf("$preprefix%02d$ppp%02d_dfq2fq_%04d_$now",$index,$pp,$list[$i]);
        }
        my $holdlist=join(',',@tmp);
  
        my $files="";
        for(my $i=0; $i<@list; $i+=$CURRENT){
            $files .=  sprintf("$maindir/$preprefix%02d.not_passed_%04d.list",$index,$list[$i]);
            $files .= " ";
        }
  
        $outputfile = sprintf("$preprefix%02d$ppp\${SGE_TASK_ID}_not_passed.list",$index+1);
#        $outputfile = sprintf("$preprefix%02d$ppp%02d.idfq.gz",$index+1,$pp);
        $command = sprintf("cat $files > $prepadir/$outputfile");
  
        $script = sprintf("$script_sub_dir/$preprefix%02d_cat_not_passed.sh",$index);
        push(@array_jobs,$script);
#        push(@allscripts,$script);
        open my $fh, ">", $script or die $!;
  
        printf $fh ("#!/bin/bash\n");
        printf $fh ("#\$ -S /bin/bash\n");
        printf $fh ("#\$ -cwd\n");
        my $jobname = sprintf("$preprefix%02d_cat_not_passed_$now",$index);
#        my $jobname = sprintf("$preprefix%02d$ppp%02d_cat_$now",$index,$pp);
        push(@second_cat_holdjids,$jobname);
        printf $fh ("#\$ -N $jobname\n");
        if($queue_req){
          printf $fh ("#\$ $queue_req\n");
        }
        printf $fh ("#\$ -o $doutdir\n");
        printf $fh ("#\$ -e $errdir\n");
        printf $fh ("#\$ -hold_jid $holdlist\n");
        printf $fh ("$command\n");
  
        close $fh;
      }
    }

    # finish: chop low (<4) depth regions and discard short (<500) reads
    {
      {
        my @tmp;
        for(my $i=0; $i<@list; ++$i){
          $tmp[$i]=sprintf("$preprefix%02d_ec_%04d_$now",$index,$list[$i]);
#          $tmp[$i]=sprintf("$preprefix%02d$ppp%02d_ec_%04d_$now",$index,$pp,$list[$i]);
        }
        my $holdlist=join(',',@tmp);
  
        for(my $i=0; $i<@list; $i+=$CURRENT){
          $outputfile = sprintf("$preprefix%02d_passed_finish_%04d.idfq.gz",$index,$list[$i]);
          $command = sprintf("gzip -d -c $maindir/$preprefix%02d_%04d.dfq.gz | $bindir/dfq2fq_v2.pl - --pnp --clc $confident_length_coefficient --cd $confident_depth | $bindir/dfq2fq_v2.pl - --finish --valid_depth $valid_depth --valid_read_length $valid_read_length | gzip -c -1 > $maindir/$outputfile", $index,$list[$i]);
          #$command = sprintf("gzip -d -c $maindir/$preprefix%02d_%04d.dfq.gz | $bindir/dfq2fq_v2.pl - -finish -valid_depth $valid_depth -valid_read_length $valid_read_length | gzip -c -1 > $maindir/$outputfile", $index,$list[$i]);
  
          $script = sprintf("$script_sub_dir/$preprefix%02d_passed_finish_%04d.sh",$index,$list[$i]);
          push(@array_jobs,$script);
#          push(@allscripts,$script);
          open my $fh, ">", $script or die $!;
  
          printf $fh ("#!/bin/bash\n");
          printf $fh ("#\$ -S /bin/bash\n");
          printf $fh ("#\$ -cwd\n");
          printf $fh ("#\$ -N $preprefix%02d_passed_finish_%04d_$now\n",$index,$list[$i]);
#          printf $fh ("#\$ -N $preprefix%02d$ppp%02d_finish_%04d_$now\n",$index,$pp,$list[$i]);
          if($queue_req){
            printf $fh ("#\$ $queue_req\n");
          }
          printf $fh ("#\$ -o $doutdir\n");
          printf $fh ("#\$ -e $errdir\n");
          if(!$opt_finish){
            printf $fh ("#\$ -hold_jid $holdlist\n");
          }
          printf $fh ("$command\n");
          for(my $j=1; $j<$CURRENT && $i+$j<@list; ++$j){
            $command = sprintf("gzip -d -c $maindir/$preprefix%02d_%04d.dfq.gz | $bindir/dfq2fq_v2.pl - --pnp --clc $confident_length_coefficient --cd $confident_depth | $bindir/dfq2fq_v2.pl - --finish --valid_depth $valid_depth --valid_read_length $valid_read_length | gzip -c -1 >> $maindir/$outputfile", $index,$list[$i+$j]);
            #$command = sprintf("gzip -d -c $maindir/$preprefix%02d_%04d.dfq.gz | $bindir/dfq2fq_v2.pl - -finish -valid_depth $valid_depth -valid_read_length $valid_read_length | gzip -c -1 >> $maindir/$outputfile", $index,$list[$i+$j]);
            printf $fh ("$command\n");
          }
  
          close $fh;
        }
      }
  
      {
        my @tmp;
        for(my $i=0; $i<@list; $i+=$CURRENT){
          $tmp[$i/$CURRENT]=sprintf("$preprefix%02d_passed_finish_%04d_$now",$index,$list[$i]);
#          $tmp[$i/$CURRENT]=sprintf("$preprefix%02d$ppp%02d_finish_%04d_$now",$index,$pp,$list[$i]);
        }
        my $holdlist=join(',',@tmp);
  
        my $files="";
        for(my $i=0; $i<@list; $i+=$CURRENT){
          $files .=  sprintf("$maindir/$preprefix%02d_passed_finish_%04d.idfq.gz",$index,$list[$i]);
          $files .= " ";
        }
  
        $outputfile = sprintf("$preprefix%02d$ppp\${SGE_TASK_ID}.passed.fin.idfq.gz",$index+1);
        $command = sprintf("cat $files > $prepadir/$outputfile");
  
        $script = sprintf("$script_sub_dir/$preprefix%02d_cat_passed_finish.sh",$index);
        push(@array_jobs,$script);
#        push(@allscripts,$script);
        open my $fh, ">", $script or die $!;
  
        printf $fh ("#!/bin/bash\n");
        printf $fh ("#\$ -S /bin/bash\n");
        printf $fh ("#\$ -cwd\n");
        my $jobname = sprintf("$preprefix%02d_cat_passed_finish_$now",$index);
#        my $jobname = sprintf("$preprefix%02d$ppp%02d_cat_finish_$now",$index,$pp);
        push(@second_cat_passed_finish_holdjids,$jobname);
        printf $fh ("#\$ -N $jobname\n");
        if($queue_req){
          printf $fh ("#\$ $queue_req\n");
        }
        printf $fh ("#\$ -o $doutdir\n");
        printf $fh ("#\$ -e $errdir\n");
        printf $fh ("#\$ -hold_jid $holdlist\n");
        printf $fh ("$command\n");
  
        close $fh;
      }
    }

    # pnp finish
    {
      {
        my @tmp;
        for(my $i=0; $i<@list; ++$i){
          $tmp[$i]=sprintf("$preprefix%02d_ec_%04d_$now",$index,$list[$i]);
#          $tmp[$i]=sprintf("$preprefix%02d$ppp%02d_ec_%04d_$now",$index,$pp,$list[$i]);
        }
        my $holdlist=join(',',@tmp);
  
        for(my $i=0; $i<@list; $i+=$CURRENT){
          $outputfile = sprintf("$preprefix%02d_pnp_finish_%04d.idfq.gz",$index,$list[$i]);
          $command = sprintf("gzip -d -c $maindir/$preprefix%02d_%04d.dfq.gz | $bindir/dfq2fq_v2.pl - --finish --valid_depth $valid_depth -valid_read_length $valid_read_length | gzip -c -1 > $maindir/$outputfile", $index,$list[$i]);
  
          $script = sprintf("$script_sub_dir/$preprefix%02d_pnp_finish_%04d.sh",$index,$list[$i]);
          push(@array_jobs,$script);
#          push(@allscripts,$script);
          open my $fh, ">", $script or die $!;
  
          printf $fh ("#!/bin/bash\n");
          printf $fh ("#\$ -S /bin/bash\n");
          printf $fh ("#\$ -cwd\n");
          printf $fh ("#\$ -N $preprefix%02d_pnp_finish_%04d_$now\n",$index,$list[$i]);
#          printf $fh ("#\$ -N $preprefix%02d$ppp%02d_finish_%04d_$now\n",$index,$pp,$list[$i]);
          if($queue_req){
            printf $fh ("#\$ $queue_req\n");
          }
          printf $fh ("#\$ -o $doutdir\n");
          printf $fh ("#\$ -e $errdir\n");
          if(!$opt_finish){
            printf $fh ("#\$ -hold_jid $holdlist\n");
          }
          printf $fh ("$command\n");
          for(my $j=1; $j<$CURRENT && $i+$j<@list; ++$j){
            $command = sprintf("gzip -d -c $maindir/$preprefix%02d_%04d.dfq.gz | $bindir/dfq2fq_v2.pl - --finish --valid_depth $valid_depth --valid_read_length $valid_read_length | gzip -c -1 >> $maindir/$outputfile", $index,$list[$i+$j]);
            printf $fh ("$command\n");
          }
  
          close $fh;
        }
      }
  
      {
        my @tmp;
        for(my $i=0; $i<@list; $i+=$CURRENT){
          $tmp[$i/$CURRENT]=sprintf("$preprefix%02d_pnp_finish_%04d_$now",$index,$list[$i]);
#          $tmp[$i/$CURRENT]=sprintf("$preprefix%02d$ppp%02d_finish_%04d_$now",$index,$pp,$list[$i]);
        }
        my $holdlist=join(',',@tmp);
  
        my $files="";
        for(my $i=0; $i<@list; $i+=$CURRENT){
          $files .=  sprintf("$maindir/$preprefix%02d_pnp_finish_%04d.idfq.gz",$index,$list[$i]);
          $files .= " ";
        }
  
        $outputfile = sprintf("$preprefix%02d$ppp\${SGE_TASK_ID}.pnp.fin.idfq.gz",$index+1);
        $command = sprintf("cat $files > $prepadir/$outputfile");
  
        $script = sprintf("$script_sub_dir/$preprefix%02d_cat_pnp_finish.sh",$index);
        push(@array_jobs,$script);
#        push(@allscripts,$script);
        open my $fh, ">", $script or die $!;
  
        printf $fh ("#!/bin/bash\n");
        printf $fh ("#\$ -S /bin/bash\n");
        printf $fh ("#\$ -cwd\n");
        my $jobname = sprintf("$preprefix%02d_cat_pnp_finish_$now",$index);
#        my $jobname = sprintf("$preprefix%02d$ppp%02d_cat_finish_$now",$index,$pp);
        push(@second_cat_pnp_finish_holdjids,$jobname);
        printf $fh ("#\$ -N $jobname\n");
        if($queue_req){
          printf $fh ("#\$ $queue_req\n");
        }
        printf $fh ("#\$ -o $doutdir\n");
        printf $fh ("#\$ -e $errdir\n");
        printf $fh ("#\$ -hold_jid $holdlist\n");
        printf $fh ("$command\n");
  
        close $fh;
      }
    }
#  print STDERR "array_jobs printed\n";

  # second cat
  {
    my $holdlist=join(',',@second_cat_holdjids);
 
    my $files="";
    for(my $pp=1; $pp<=$pre_partition; ++$pp){
      $files .=  sprintf("$prepadir/$preprefix%02d$ppp%d_not_passed.list",$index+1,$pp);
      $files .= " ";
    }

    $outputfile = sprintf("$preprefix%02d_not_passed.list",$index+1);
    $command = sprintf("cat $files | sort -n - > $datadir/$outputfile.tmp && mv $datadir/$outputfile.tmp $datadir/$outputfile");

    $script = sprintf("$script_sub_dir/$preprefix%02d_second_cat_not_passed.sh",$index);
    push(@post_array_jobs,$script);
#    push(@allscripts,$script);
    open my $fh, ">", $script or die $!;

    printf $fh ("#!/bin/bash\n");
    printf $fh ("#\$ -S /bin/bash\n");
    printf $fh ("#\$ -cwd\n");
    my $jobname = sprintf("$preprefix%02d_second_cat_not_passed_$now",$index);
    printf $fh ("#\$ -N $jobname\n");
    if($queue_req){
      printf $fh ("#\$ $queue_req\n");
    }
    printf $fh ("#\$ -o $datadir\n");
    printf $fh ("#\$ -e $datadir\n");
    printf $fh ("#\$ -hold_jid $holdlist\n");
    printf $fh ("$command\n");

    # extract
    my $next_not_passed = sprintf("$datadir/$preprefix%02d.not_passed.idfq.gz",$index+1);
    $command = "gzip -d -c $orig_idfq | $bindir/extract_fq.pl - -include $datadir/$outputfile | gzip -c -1 - > $next_not_passed";
    printf $fh ("$command\n");

    close $fh;
  }
  {
    my $holdlist=join(',',@second_cat_passed_finish_holdjids);
 
    my $files="";
    for(my $pp=1; $pp<=$pre_partition; ++$pp){
      $files .=  sprintf("$prepadir/$preprefix%02d$ppp%d.passed.fin.idfq.gz",$index+1,$pp);
      $files .= " ";
    }

    $outputfile = sprintf("$preprefix%02d.passed.fin.idfq.gz",$index+1);
    $command = sprintf("cat $files > $datadir/$outputfile.tmp && mv $datadir/$outputfile.tmp $datadir/$outputfile");

    $script = sprintf("$script_sub_dir/$preprefix%02d_second_cat_passed_finish.sh",$index);
    push(@post_array_jobs,$script);
#    push(@allscripts,$script);
    open my $fh, ">", $script or die $!;

    printf $fh ("#!/bin/bash\n");
    printf $fh ("#\$ -S /bin/bash\n");
    printf $fh ("#\$ -cwd\n");
    my $jobname = sprintf("$preprefix%02d_second_cat_passed_finish_$now",$index);
    printf $fh ("#\$ -N $jobname\n");
    if($queue_req){
      printf $fh ("#\$ $queue_req\n");
    }
    printf $fh ("#\$ -o $datadir\n");
    printf $fh ("#\$ -e $datadir\n");
    printf $fh ("#\$ -hold_jid $holdlist\n");
    printf $fh ("$command\n");

    close $fh;
  }
  my @get_top20x_holdjids=();
  my $get_top20x_inputfile;
  {
    my $holdlist=join(',',@second_cat_pnp_finish_holdjids);
 
    my $files="";
    for(my $pp=1; $pp<=$pre_partition; ++$pp){
      $files .=  sprintf("$prepadir/$preprefix%02d$ppp%d.pnp.fin.idfq.gz",$index+1,$pp);
      $files .= " ";
    }

    $outputfile = sprintf("$preprefix%02d.fin.idfq.gz",$index+1);
    $get_top20x_inputfile = "$datadir/$outputfile";
    $command = sprintf("cat $files $passed_all > $datadir/$outputfile.tmp && mv $datadir/$outputfile.tmp $datadir/$outputfile");

    $script = sprintf("$script_sub_dir/$preprefix%02d_second_cat_pnp_finish.sh",$index);
    push(@post_array_jobs,$script);
#    push(@allscripts,$script);
    open my $fh, ">", $script or die $!;

    printf $fh ("#!/bin/bash\n");
    printf $fh ("#\$ -S /bin/bash\n");
    printf $fh ("#\$ -cwd\n");
    my $jobname = sprintf("$preprefix%02d_second_cat_pnp_finish_$now",$index);
    push(@get_top20x_holdjids,$jobname);
    printf $fh ("#\$ -N $jobname\n");
    if($queue_req){
      printf $fh ("#\$ $queue_req\n");
    }
    printf $fh ("#\$ -o $datadir\n");
    printf $fh ("#\$ -e $datadir\n");
    printf $fh ("#\$ -hold_jid $holdlist\n");
    printf $fh ("$command\n");

    close $fh;
  }
  my @fastqToCA_holdjids=();
  my $fastqToCA_input;
  if(!$opt_ec_only)
  {
    my $holdlist=join(',',@get_top20x_holdjids);
    $outputfile = sprintf("$preprefix%02d.fin.top20x.fq",$index+1);
    $fastqToCA_input = "$datadir/$outputfile";

    my $PG1 = "$bindir/get_top_20x_fa.pl";
    $command = sprintf("gzip -d -c $get_top20x_inputfile | $PG1 - -g $estimated_genome_size -q > $datadir/$outputfile");

    $script = sprintf("$script_sub_dir/$preprefix%02d_get_top20x.sh",$index);
    push(@post_array_jobs,$script);
    open my $fh, ">", $script or die $!;

    printf $fh ("#!/bin/bash\n");
    printf $fh ("#\$ -S /bin/bash\n");
    printf $fh ("#\$ -cwd\n");
    my $jobname = sprintf("$preprefix%02d_get_top20x_$now",$index);
    push @fastqToCA_holdjids,$jobname;
    printf $fh ("#\$ -N $jobname\n");
    if($queue_req){
      printf $fh ("#\$ $queue_req\n");
    }
    printf $fh ("#\$ -o $datadir\n");
    printf $fh ("#\$ -e $datadir\n");
    printf $fh ("#\$ -hold_jid $holdlist\n");
    printf $fh ("$command\n");

    close $fh;
  }
  my @runCA_holdjids=();
  my $runCA_input;
  if(!$opt_ec_only)
  {
    my $holdlist=join(',',@fastqToCA_holdjids);
    $outputfile = sprintf("$preprefix%02d.fin.top20x.frg",$index+1);
    $runCA_input = "$datadir/$outputfile";

    my $PG1 = "$ca_path/fastqToCA";
    $command = sprintf("$PG1 -libraryname foo -technology pacbio -reads $fastqToCA_input > $datadir/$outputfile");

    $script = sprintf("$script_sub_dir/$preprefix%02d_fastqToCA.sh",$index);
    push(@post_array_jobs,$script);
    open my $fh, ">", $script or die $!;

    printf $fh ("#!/bin/bash\n");
    printf $fh ("#\$ -S /bin/bash\n");
    printf $fh ("#\$ -cwd\n");
    my $jobname = sprintf("$preprefix%02d_fastqToCA_$now",$index);
    push @runCA_holdjids,$jobname;
    printf $fh ("#\$ -N $jobname\n");
    if($queue_req){
      printf $fh ("#\$ $queue_req\n");
    }
    printf $fh ("#\$ -o $datadir\n");
    printf $fh ("#\$ -e $datadir\n");
    printf $fh ("#\$ -hold_jid $holdlist\n");
    printf $fh ("$command\n");

    close $fh;
  }
  if(!$opt_ec_only)
  {
    my $holdlist=join(',',@runCA_holdjids);

    my $PG1 = "$ca_path/runCA";
    $command = sprintf("$PG1 -dir $datadir/${now}_c%02d.fin.top20x -p ${now}_c%02d.fin.top20x -s $asm_spec $runCA_input |& tee -a $datadir/do_${now}_c%02d.fin.top20x.log",$index+1,$index+1,$index+1);

    $script = sprintf("$script_sub_dir/$preprefix%02d_runCA.sh",$index);
    push(@post_array_jobs,$script);
    open my $fh, ">", $script or die $!;

    printf $fh ("#!/bin/bash\n");
    printf $fh ("#\$ -S /bin/bash\n");
    printf $fh ("#\$ -cwd\n");
    my $jobname = sprintf("$preprefix%02d_runCA_$now",$index);
    printf $fh ("#\$ -N $jobname\n");
    if($queue_req){
      printf $fh ("#\$ $queue_req\n");
    }
    printf $fh ("#\$ -o $datadir\n");
    printf $fh ("#\$ -e $datadir\n");
    printf $fh ("#\$ -hold_jid $holdlist\n");
    printf $fh ("$command\n");

    close $fh;
  }

#  print STDERR "post_array_jobs printed\n";

  if(!$opt_dryrun){
    my $require = "$datadir/".sprintf("$preprefix%02d.not_passed.idfq.gz",$index);
    #my $require_2 = sprintf("$datadir/$preprefix%02d.passed.fin.idfq.gz",$index+1);
    while(!-e $require && $index>0){
      `sleep 10`;
      my $errdir = sprintf("$datadir/$preprefix%02d/err/",$index-1);
      my $doutdir = sprintf("$datadir/$preprefix%02d/dout/",$index-1);
      my @errfiles = split /\n/,`ls $errdir`;
      for(my $i=0; $i<@errfiles; ++$i){
        my $target = "$errdir/$errfiles[$i]";
        if(-s $target == 0){
          `rm $target`;
        }
        else{
          printf STDERR "$target not deleted\n";
        }
      }
      my @doutfiles = split /\n/,`ls $doutdir`;
      for(my $i=0; $i<@doutfiles; ++$i){
        my $target = "$doutdir/$doutfiles[$i]";
        if(-s $target == 0){
          `rm $target`;
        }
        else{
          printf STDERR "$target not deleted\n";
        }
      }
    }
    my $task_f=1;
    my $task_l=$pre_partition;
    if($task_l<1){
      die "task_l must be >= 1\n";
    }
    for(my $i=0; $i<@pre_array_jobs; ++$i){
      `qsub $pre_array_jobs[$i]`;
    }
    for(my $i=0; $i<@array_jobs; ++$i){
      `qsub -t $task_f-$task_l:1 $array_jobs[$i]`;
    }
    for(my $i=0; $i<@post_array_jobs; ++$i){
      `qsub $post_array_jobs[$i]`;
    }
  }
}

if(!$opt_dryrun)
{
  my $index = $to-1;
#  my $require = "$datadir/".sprintf("$preprefix%02d.idfq.gz",$index+1);
  my $require = "$datadir/".sprintf("$preprefix%02d.fin.idfq.gz",$index+1);
  my $errdir = sprintf("$datadir/$preprefix%02d/err/",$index);
  my $doutdir = sprintf("$datadir/$preprefix%02d/dout/",$index);
  while(!-e $require){
    `sleep 10`;
    my @errfiles = split /\n/,`ls $errdir`;
    for(my $i=0; $i<@errfiles; ++$i){
      my $target = "$errdir/$errfiles[$i]";
      if(-s $target == 0){
        `rm $target`;
      }
      else{
        printf STDERR "$target not deleted\n";
      }
    }
    my @doutfiles = split /\n/,`ls $doutdir`;
    for(my $i=0; $i<@doutfiles; ++$i){
      my $target = "$doutdir/$doutfiles[$i]";
      if(-s $target == 0){
        `rm $target`;
      }
      else{
        printf STDERR "$target not deleted\n";
      }
    }
  }
}
#if(!$opt_dryrun){
#  for(my $i=0; $i<@allscripts; ++$i){
#    `qsub $allscripts[$i]`;
#  }
#}


# runCA
#my $ca_command="ca_ikki_v3.pl ";
#my $ikki_log="$pwd/ca_ikki_v3.log";
#$ca_command .= "-d $datadir -from $from -to $to $asm_spec $estimated_genome_size -ca_path $ca_path > $ikki_log 2>&1";
#if($opt_dryrun){
#  printf("%s\n",$ca_command);
#}
#else{
#  `$ca_command`;
#}

