#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;

my $DEVEL;
my $DEBUG;

my $from=0;
my $loop=1;
my $partition=12;
my $word_size="";
my $evalue=1e-50;
#my $second_evalue=1e-307;
my $valid_voters=11;
my $trim=42;

my $preprefix="c";
my $CURRENT=12;

my $input="";

my $outfmt = "7 qseqid sstart send sacc qstart qend bitscore evalue pident qseq sseq";
#my $maxfilesize="100GB";

my $index=0;
my $command="";

my $valid_depth = 4;
my $valid_read_length = 500;
my $confident_depth = $valid_depth;
my $confident_length_coefficient = 0.75;

my $blasr_path="";
my $blasr_opt="";

GetOptions("from=i" => \$from, "to=i" => \$loop, "partition=i" => \$partition, "evalue=f" => \$evalue, "valid_voters=i" => \$valid_voters, "trim=i" => \$trim, "input=s" => \$input, "devel"=>\$DEVEL, "debug"=>\$DEBUG, "word_size=i"=>\$word_size,'blasr_path=s'=>\$blasr_path,'blasr_opt=s'=>\$blasr_opt);
#GetOptions("from=i" => \$from, "to=i" => \$loop, "partition=i" => \$partition, "evalue=f" => \$evalue, "second_evalue=f" => \$second_evalue, "valid_voters=i" => \$valid_voters, "trim=i" => \$trim, "input=s" => \$input, "devel"=>\$DEVEL, "debug"=>\$DEBUG, "word_size=i"=>\$word_size,'blasr_path=s'=>\$blasr_path,'blasr_opt=s'=>\$blasr_opt);

printf STDERR ("#>- params -<#\n");
printf STDERR ("input\t%s\n",$input);
printf STDERR ("from\t%s\n",$from);
printf STDERR ("to\t%s\n",$loop);
printf STDERR ("partition\t%s\n",$partition);
if($word_size){
  printf STDERR ("word_size\t%d\n",$word_size);
}
printf STDERR ("evalue\t%g\n",$evalue);
#printf STDERR ("second_evalue\t%g\n",$second_evalue);
printf STDERR ("valid_voters\t%s\n",$valid_voters);
printf STDERR ("trim\t%d\n",$trim);
if($DEVEL){
  printf STDERR ("development_mode\t%s\n","true");
}
if($blasr_path){
  printf STDERR ("blasr_path\t%s\n",$blasr_path);
  if($blasr_opt){
    printf STDERR ("blasr_opt\t%s\n",$blasr_opt);
  }
}
printf STDERR ("#>- params -<#\n");
#exit;
if($from == 0 && !$input){
  die "give an input fastq file\n";
}

my $date="";
my $message="";

my $iprefix = sprintf("$preprefix%02d",0);
my $orig_idfq = "$iprefix.idfq.gz";

if($from == 0)
{
  # fq -> idfq (& id2n)
  my $PG = "fq2idfq.pl";
  if($DEVEL){
    $PG = "./".$PG;
  }
  my $not_passed_all = sprintf("$preprefix%02d.not_passed.idfq.gz",0);
  my $command = "$PG $input --prefix $iprefix | gzip -c -1 > $orig_idfq && ln -sf $orig_idfq $not_passed_all";
  $date = `date`;
  chomp $date;
  printf("%s\n",$date);
  $message = $command;
  $message =~ s/\&/\&\n/g;
  printf("stt:\n %s\n",$message);

  `$command`;

  $date = `date`;
  chomp $date;
  printf("%s\n",$date);
  $message = $command;
  $message =~ s/\&/\&\n/g;
  printf("end:\n %s\n",$message);
}

for(my $index=$from; $index<$loop; ++$index){
  if($word_size){
    if($word_size == 4){
      #$to = $index;
      last;
    }
    my $foo = 1.0;
    for(my $i=0; $i < ($index-$from); ++$i){
      $foo = $foo*1.0/2.0;
    }
    $word_size = int(($word_size+0.0)*$foo);
    if($word_size < 4){
      $word_size = 4;
    }
  }
#  printf STDERR ("word_size %d\n",$word_size);
  {
    my $command1="";
    my $PG1 = "dfq2fq_v2.pl";
    if($DEVEL){
      $PG1 = "./".$PG1;
    }
    my $passed_all_before_this_cycle="";
    for(my $i=$from; $i<=$index; ++$i){
      my $passed = sprintf("$preprefix%02d.passed.fin.idfq.gz",$i);
      if(-e $passed){
        $passed_all_before_this_cycle .= $passed." ";
      }
      else{
        if($i > 0){
          die "$passed does NOT exitst\n";
        }
      }
    }
    my $not_passed = sprintf("$preprefix%02d.not_passed.idfq.gz",$index);
    if($blasr_path){
      $command1 .= sprintf("cat $passed_all_before_this_cycle $not_passed | gzip -d -c - | $PG1 -f - > $preprefix%02d.fasta ",$index);
      $command1 .= sprintf(" && $blasr_path/sawriter $preprefix%02d.fasta ",$index);
    }
    else{
      $command1.= sprintf("cat $passed_all_before_this_cycle $not_passed | gzip -d -c - | $PG1 -f - | makeblastdb -in - -dbtype nucl -out $preprefix%02d -title $preprefix%02d ",$index,$index);
    }
    if($index > 0){
      # do NOT make db.
      $command1 = "sleep 1";
    }

    my $PG2="partition_fa.pl";
    if($DEVEL){
      $PG2 = "./".$PG2;
    }
    my $command2="";
    $command2 .= sprintf("cat $not_passed | gzip -d -c - | $PG1 -f - | $PG2 - $partition -p $preprefix%02d", $index);

    $date = `date`;
    chomp $date;
    printf("%s\n",$date);
    $message = $command1;
    $message =~ s/\&/\&\n/g;
    printf("stt:\n %s\n",$message);
    $message = $command2;
    $message =~ s/\&/\&\n/g;
    printf("stt:\n %s\n",$message);
    `$command1 & $command2 & wait`;
    $date = `date`;
    chomp $date;
    printf("%s\n",$date);
    $message = $command1;
    $message =~ s/\&/\&\n/g;
    printf("end:\n %s\n",$message);
    $message = $command2;
    $message =~ s/\&/\&\n/g;
    printf("end:\n %s\n",$message);
  }

  {
    $command="";
    my $tmp_evalue;
    if($index >= 0){
      $tmp_evalue = $evalue;
    }
#    elsif($index > 0){
#      $tmp_evalue = $second_evalue;
#    }
    else{
      die "strange index: $index (index must be >= 0)\n";
    }
    for(my $i=0; $i<$partition; ++$i){
      my $PG2=sprintf("bfmt72s -c $trim -u -i");
      #my $PG2=sprintf("bfmt72s -u -i -q $preprefix%02d.idfq.gz",$index);
      if($DEVEL){
        $PG2 = "./".$PG2;
      }
      my $PG3="nss2v -e -v $valid_voters";
      #my $PG3="nss2v -e -v $valid_voters -c $trim -q";
      if($DEVEL){
        $PG3 = "./".$PG3;
      }
      my $PG4="myrealigner -f -B $valid_voters -b 3";
      if($DEVEL){
        $PG4 = "./".$PG4;
      }
      my $BLASTN = "blastn";
      if($word_size){
        $BLASTN .= " -word_size $word_size";
      }
      if($DEBUG){
        die "sorry. under construction\n";
        $PG4="myrealigner -B $valid_voters -b 3";
        if($DEVEL){
          $PG4 = "./".$PG4;
        }
        my $PG5="myrealigner -f -n -B $valid_voters -b 3";
        if($DEVEL){
          $PG5 = "./".$PG5;
        }
        $command.= sprintf("cat $preprefix%02d_%04d.fa | $BLASTN -db $preprefix%02d -query - -evalue $tmp_evalue -outfmt '$outfmt' | tee $preprefix%02d_%04d.blastn | $PG2 - | tee $preprefix%02d_%04d.nss | $PG3 - | tee $preprefix%02d_%04d.vertical | $PG4 - | tee $preprefix%02d_%04d.realigned | $PG5 - |  gzip -c -1 > $preprefix%02d_%04d.dfq.gz & ", $index,$i,$index,$index,$i,$index,$i,$index,$i,$index,$i,$index,$i);
      }
      else{
        if($blasr_path){
          $command .= sprintf("$blasr_path/blasr $preprefix%02d_%04d.fa $preprefix%02d.fasta -sa $preprefix%02d.fasta.sa -m 5 $blasr_opt -out tmp%02d_%04d.m5 ",$index,$i,0,0,$index,$i);
          $command .= sprintf(" && cat tmp%02d_%04d.m5 | m52bfmt7 - |  $PG2 - | $PG3 - | $PG4 - | gzip -c -1 > $preprefix%02d_%04d.dfq.gz & ", $index,$i,$index,$i);
        }
        else{
          $command.= sprintf("cat $preprefix%02d_%04d.fa | $BLASTN -db $preprefix%02d -query - -evalue $tmp_evalue -outfmt '$outfmt' | $PG2 - | $PG3 - | $PG4 - | gzip -c -1 > $preprefix%02d_%04d.dfq.gz & ", $index,$i,0,$index,$i);
        }
      }
    }
    $command .= " wait ";

    $date = `date`;
    chomp $date;
    printf("%s\n",$date);
    $message = $command;
    $message =~ s/\&/\&\n/g;
    printf("stt:\n %s\n",$message);
    `$command`;
    $date = `date`;
    chomp $date;
    printf("%s\n",$date);
    printf("end:\n %s\n",$message);
  }

  # passed
  {
    my $current=$CURRENT;

    for(my $i=0; $i<$partition; $i+=$current){
      $command="";
      my $PG1 = "dfq2fq_v2.pl";
      if($DEVEL){
        $PG1 = "./".$PG1;
      }
      my $out = sprintf("$preprefix%02d.passed_%04d.fin.idfq.gz",$index,$i);
      if(-e $out){
        `rm $out`;
      }
      for(my $j=$i; $j<$i+$current && $j<$partition; ++$j){
        $command = sprintf("gzip -d -c $preprefix%02d_%04d.dfq.gz | $PG1 - --pnp --clc $confident_length_coefficient --cd $confident_depth | $PG1 - --finish --valid_depth $valid_depth --valid_read_length $valid_read_length | gzip -c -1 >> $out", $index,$j);
        `$command`;
      }

      #$date = `date`;
      #chomp $date;
      #printf("%s\n",$date);
      #$message = $command;
      #$message =~ s/\&/\&\n/g;
      #printf("stt:\n %s\n",$message);
      #`$command`;
      #$date = `date`;
      #chomp $date;
      #printf("%s\n",$date);
      #printf("end:\n %s\n",$message);
    }

    {
      $date = `date`;
      chomp $date;
      printf("%s\n",$date);
      printf("stt:\n %s\n","cat");
      $command="";
      my $outputfile = sprintf("$preprefix%02d.passed.fin.idfq.gz",$index+1);

      my $current=$CURRENT;
      my $files="";
      for(my $i=0; $i<$partition; $i+=$current){
        $files .=  sprintf(" $preprefix%02d.passed_%04d.fin.idfq.gz",$index,$i);
      }
      $command = sprintf("cat $files > $outputfile");
      `$command`;
      $date = `date`;
      chomp $date;
      printf("%s\n",$date);
      printf("end:\n %s\n","cat");
    }
  }

  # NOT passed
  {
    my $current=$CURRENT;
#    my $valid_depth = 4;
#    my $valid_read_length = 500;
#    my $confident_depth = $valid_depth;
#    my $confident_length_coefficient = 0.75;

    for(my $i=0; $i<$partition; $i+=$current){
      $command="";
      my $PG1 = "dfq2fq_v2.pl";
      if($DEVEL){
        $PG1 = "./".$PG1;
      }
      my $nplist = sprintf("$preprefix%02d.not_passed_%04d.list",$index,$i);
      if(-e $nplist){
        `rm $nplist`;
      }
      for(my $j=$i; $j<$i+$current && $j<$partition; ++$j){
        $command = sprintf("gzip -d -c $preprefix%02d_%04d.dfq.gz | $PG1 - --nlist --clc $confident_length_coefficient --cd $confident_depth >> $nplist ",$index,$j);
        `$command`;
      }
      #$command .= " wait ";

      $date = `date`;
      chomp $date;
      printf("np1 %s\n",$date);
      #$message = $command;
      #$message =~ s/\&/\&\n/g;
      #printf("stt:\n %s\n",$message);
      #`$command`;
      #$date = `date`;
      #chomp $date;
      #printf("%s\n",$date);
      #printf("end:\n %s\n",$message);
    }

    my $PG2 = "extract_fq.pl";
    if($DEVEL){
      $PG2 = "./".$PG2;
    }
    my $nlist_all = sprintf("$preprefix%02d.nlistall.tmp",$index);
    my $nlist_all_sorted = sprintf("$preprefix%02d.nlistall",$index);
#    if(-e $nlist_all){
#      `rm $nlist_all`;
#    }
    my $files="";
    for(my $i=0; $i<$partition; $i+=$current){
      $files .=  sprintf(" $preprefix%02d.not_passed_%04d.list",$index,$i);
    }
    $command = sprintf("cat $files > $nlist_all");
    `$command`;
    `sort -n $nlist_all > $nlist_all_sorted`;
#    `mv $nlist_all $nlist_all_sorted`;
    my $not_passed_all = sprintf("$preprefix%02d.not_passed.idfq.gz",$index+1);
    `gzip -d -c $orig_idfq | $PG2 - -include $nlist_all_sorted | gzip -c -1 - > $not_passed_all`;
    $date = `date`;
    chomp $date;
    printf("extract_fq %s\n",$date);
  }

=pod
  {
    my $current=$CURRENT;
    for(my $i=0; $i<$partition; $i+=$current){
      $command="";
      my $PG1 = "dfq2fq_v2.pl";
      if($DEVEL){
        $PG1 = "./".$PG1;
      }
      for(my $j=$i; $j<$i+$current && $j<$partition; ++$j){
        $command.= sprintf("gzip -d -c $preprefix%02d_%04d.dfq.gz | $PG1 - | gzip -c -1 > $preprefix%02d.refined_%04d.idfq.gz & ", $index,$j,$index,$j);
      }
      $command .= " wait ";

      $date = `date`;
      chomp $date;
      printf("%s\n",$date);
      $message = $command;
      $message =~ s/\&/\&\n/g;
      printf("stt:\n %s\n",$message);
      `$command`;
      $date = `date`;
      chomp $date;
      printf("%s\n",$date);
      printf("end:\n %s\n",$message);
    }
  }

  {
    $date = `date`;
    chomp $date;
    printf("%s\n",$date);
    printf("stt:\n %s\n","cat");
    $command="";
    my $outputfile = sprintf("$preprefix%02d.idfq.gz",$index+1);
    $command.="touch $outputfile; rm $outputfile;";
    `$command`;

    my $current=$CURRENT;
    for(my $i=0; $i<$partition; $i+=$current){
      my $files="";
      for(my $j=$i; $j<$i+$current && $j<$partition; ++$j){
        $files .=  sprintf(" $preprefix%02d.refined_%04d.idfq.gz",$index,$j);
      }
      $command="";
      $command .= sprintf("cat $files >> $outputfile");
      `$command`;
    }
    $date = `date`;
    chomp $date;
    printf("%s\n",$date);
    printf("end:\n %s\n","cat");
  }
=cut
  #finish
  {
    my $passed_all_before_this_cycle="";
    for(my $i=$from; $i<=$index; ++$i){
      my $passed = sprintf("$preprefix%02d.passed.fin.idfq.gz",$i);
      if(-e $passed){
        $passed_all_before_this_cycle .= $passed." ";
      }
      else{
        if($i > 0){
          die "$passed does NOT exitst\n";
        }
      }
    }
  
    my $the_rest = sprintf("$preprefix%02d.pnp.fin.idfq.gz",$index+1);
  
    {
      #$date = `date`;
      #chomp $date;
      #printf("%s\n",$date);
      #printf("stt:\n %s\n","cat");
      #$command="";
  
      my $PG = "dfq2fq_v2.pl";
      my $current = $CURRENT;
      for(my $i=0; $i<$partition; $i+=$current){
        my $out = sprintf("$preprefix%02d.refined_%04d.fin.idfq.gz",$index,$i);
        if(-e $out){
          `rm $out`;
        }
        for(my $j=$i; $j<$i+$current; ++$j){
          my $in = sprintf("$preprefix%02d_%04d.dfq.gz",$index,$j);
          $command = sprintf("gzip -d -c $in | $PG - --finish --valid_depth $valid_depth --valid_read_length $valid_read_length | gzip -c -1 >> $out");
          `$command`;
        }
      }
  
      # merge
      my $files="";
      for(my $i=0; $i<$partition; $i+=$current){
        $files .=  sprintf(" $preprefix%02d.refined_%04d.fin.idfq.gz",$index,$i);
      }
      `cat $files > $the_rest`;
      #$date = `date`;
      #chomp $date;
      #printf("%s\n",$date);
      #printf("end:\n %s\n","cat");
    }

    my $finished_file = sprintf("$preprefix%02d.fin.idfq.gz",$index+1);
    `cat $passed_all_before_this_cycle $the_rest > $finished_file`;
    $date = `date`;
    chomp $date;
    printf("finish %s\n",$date);
  }
}
