#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;

my $DEVEL;
my $DEBUG;

my $opt_dryrun;
my $opt_ec_only;
GetOptions("n" => \$opt_dryrun,"devel" => \$DEVEL,"debug"=>\$DEBUG,"ec_only"=>\$opt_ec_only);

my %params;

if(@ARGV != 2){
  die "USAGE: <this> <ec.spec> <asm.spec>\n\t[-n: dryrun]\n";
  #die "USAGE: <this> estimated_genome_size(integer>0) [-i integer (iterations)]\n";
}

my $spec=$ARGV[1];

{
  my $ec_spec = $ARGV[0];
  open my $fh,"<",$ec_spec or die "cannot open $ec_spec :$!\n";
  while($_ = <$fh>){
    next if($_ =~ /^\s+$/);
    next if($_ =~ /^\s*#/);
    chomp;
    my @line = split /\s+/,$_;
  #  if(@line < 2){
  #    die "strange line in $ec_spec\n$_\n";
  #  }
    for(my $i=0; $i<@line; ++$i){
      if($line[$i] =~ /^\s*#/){
        @line = @line[0..$i-1];
        last;
      }
    }
    $params{$line[0]}=join(" ",@line[1..@line-1]);
    if($params{$line[0]}){
  #    printf("%s %s#\n",$line[0],$params{$line[0]});
    }
  }
  close $fh;
}

my $input_fastq;
my $from=0;
my $to=1;
my $partition=12;
my $evalue=1e-307;
#my $second_evalue=1e-307;
my $valid_voters=11;
my $trim=42;
my $estimated_genome_size=0;
my $ca_path="/home/imai/wgs-7.0/Linux-amd64/bin/";
my $word_size=0;

my $blasr_path="";
my $blasr_opt="";

my $rakuraku="ezez_v4.pl";
my $log="./ezez_v4.log";
#my $finish="finish_v3.pl";
#my $finish_log="./finish_v3.log";
my $ikki="ca_ikki_v3.pl";
my $ikki_log="./ca_ikki_v3.log";
#my $spec="./asm.spec";

if(defined($params{input_fastq})){
  $input_fastq = $params{input_fastq};
  if(!-e $input_fastq){
    die "$input_fastq does not exist.\n";
  }
}
else{
  die "specify input_fastq in ec.spec\n";
}

if(defined($params{estimated_genome_size})){
  $estimated_genome_size = $params{estimated_genome_size};
}
else{
  die "specify estimated_genome_size in ec.spec\n";
}
if($estimated_genome_size<=0){
  die "estimated_genome_size must be > 0\n";
}

if(defined($params{from})){
  $from = $params{from};
}
if(defined($params{to})){
  $to = $params{to};
}
if(defined($params{partition})){
  $partition = $params{partition};
}
if(defined($params{evalue})){
  $evalue = $params{evalue};
}
#if(defined($params{second_evalue})){
#  $second_evalue = $params{second_evalue};
#}
if(defined($params{valid_voters})){
  $valid_voters = $params{valid_voters};
}
else{
#  my $fq2fa="fq2fa.pl";
#  if($DEVEL){
#    $fq2fa="./".$fq2fa;
#  }
#  my $n_base=`$fq2fa $input_fastq | grep -v \"^>\" | tr -d \"\\n\" | wc -m`;
#  chomp $n_base;

#  $valid_voters = 11;

  my $n_base = -s $input_fastq;
  $n_base /= 2;
  $valid_voters = int(0.8*$n_base/$estimated_genome_size);
  $valid_voters = ($valid_voters < 11) ? 11 : $valid_voters;
}
if(defined($params{trim})){
  $trim = $params{trim};
}
if(defined($params{ca_path})){
  $ca_path = $params{ca_path};
}
if(defined($params{word_size})){
  $word_size = $params{word_size};
}
if(defined($params{blasr_path})){
  $blasr_path = $params{blasr_path};
}
if(defined($params{blasr_opt})){
  $blasr_opt = $params{blasr_opt};
}

printf STDERR ("#>- params -<#\n");
printf STDERR ("input_fastq %s\n",$input_fastq);
printf STDERR ("estimated_genome_size %g\n",$estimated_genome_size);
printf STDERR ("from %s\n",$from);
printf STDERR ("to %s\n",$to);
printf STDERR ("partition %s\n",$partition);
printf STDERR ("evalue %g\n",$evalue);
#printf STDERR ("second_evalue %g\n",$second_evalue);
printf STDERR ("valid_voters %s\n",$valid_voters);
printf STDERR ("trim %d\n",$trim);
printf STDERR ("ca_path %s\n",$ca_path);
if($word_size){
  printf STDERR ("word_size %d\n",$word_size);
}
if($blasr_path){
  printf STDERR ("blasr_path %s\n",$blasr_path);
  if($blasr_opt){
    printf STDERR ("blasr_opt %s\n",$blasr_opt);
  }
}
printf STDERR ("#>- params -<#\n");

if($DEVEL){
  $rakuraku = "./".$rakuraku;
  #$finish = "./".$finish;
  $ikki = "./".$ikki;
}

`which $rakuraku 1>/dev/null 2>&1`;
if($? != 0){
  die "$rakuraku does not exist. $?\n";
}

#`which $finish 1>/dev/null 2>&1`;
#if($? != 0){
#  die "$finish does not exist. $?\n";
#}

`which $ikki 1>/dev/null 2>&1`;
if($? != 0){
  die "$ikki does not exist. $?\n";
}

#my $ca_command_1="ln -s $ca_path";
#if(!-e "bin"){
#  `$ca_command_1`;
#}

if(!-e $ca_path){
  die "$ca_path does not exist.\n";
}

if(!-e $spec){
  die "$spec does not exist.\n";
}

if(!-e $input_fastq){
  die "$input_fastq does not exist.\n";
}

my $ec_command="$rakuraku ";
if($DEVEL){
  $ec_command .= "-devel ";
}
if($DEBUG){
  $ec_command .= "-debug ";
}
if($word_size){
  $ec_command .= "-word_size $word_size ";
}
if($blasr_path){
  $ec_command .= "-blasr_path $blasr_path ";
  if($blasr_opt){
    $ec_command .= "-blasr_opt \"$blasr_opt\" ";
  }
}
$ec_command .= "-input $input_fastq -from $from -to $to -partition $partition -evalue $evalue -valid_voters $valid_voters -trim $trim > $log 2>&1";
#$ec_command .= "-input $input_fastq -from $from -to $to -partition $partition -evalue $evalue -second_evalue $second_evalue -valid_voters $valid_voters -trim $trim > $log 2>&1";
#my $finish_command="$finish ";
if($DEVEL){
#  $finish_command .= "-devel ";
}
#$finish_command .= "-from $from -to $to -partition $partition > $finish_log 2>&1";
#my $ca_command_1="ln -s $ca_path";
my $ca_command_2="$ikki ";
if($DEVEL){
  $ca_command_2 .= "-devel ";
}
if($ca_path){
  $ca_command_2 .= "-ca_path $ca_path ";
}
$ca_command_2 .= "-from $from -to $to $spec $estimated_genome_size > $ikki_log 2>&1";
if($opt_dryrun){
  printf("%s\n",$ec_command);
#  printf("%s\n",$finish_command);
#  printf("%s\n",$ca_command_1);
  if(!$opt_ec_only){
    printf("%s\n",$ca_command_2);
  }
  exit;
}
`$ec_command`;
#`$finish_command`;
#`$ca_command_1`;
if(!$opt_ec_only){
  `$ca_command_2`;
}
#`$rakuraku -input=$input_fastq -from=$from -to=$to -partition=$partition -evalue=$evalue -valid_voters=$valid_voters > $log 2>&1`;
#`$finish -from=$from -to=$to -partition=$partition > $finish_log 2>&1`;
##chdir 'finish' or die "$!\n";
##`cp ~/port/ca -rd .`;
#`ln -s $ca_path`;
##chdir 'ca' or die "$!\n";
#`$ikki -i $to $spec $estimated_genome_size > $ikki_log 2>&1`;

