#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;

my $windowsize=500;
my $coverage=20;
my $opt_fastq=0;

my $estimated_genome_size = -1;
my $error_message ="USAGE <this> <in.fa> -g estimated_genome_size(integer>0)\n\t*each record must be in one line.\n\t[-w windos_size(integer>0)(500)]\n\t[-c coverage(integer>0)(20)]\n\t[-q (regards input as fq)]";

GetOptions('g=f'=>\$estimated_genome_size, 'w=i'=>\$windowsize, 'c=i'=>\$coverage, 'q' => \$opt_fastq);

if(@ARGV != 1){
  die "$error_message\n";
}

if($estimated_genome_size <= 0){
  die "$error_message\n";
}
if($coverage <= 0){
  die "$error_message\n";
}

my $header = ">";
if($opt_fastq){
  $header = "\@";
}

my %reads;
my %others;
my %qvs;

my $lines=0;

while(my $buf=<>){
  ++$lines;
  chomp $buf;
  if($buf =~ /^$header/){
    $buf =~ s/^$header//;
    my $name = $buf;
    # confirm $name was not added
    if(exists($reads{$name})){
      printf STDERR ("WARNING: the record %s is ambiguous.\n",$name);
    }
    $buf=<>;
    ++$lines;
    chomp $buf;
    $reads{$name} = $buf;
    if($opt_fastq){
      $buf = <>;# +
      ++$lines;
      chomp $buf;
      $others{$name} = $buf;
      #printf("%s\n",$buf);
      #printf("%s\n",$others{$name});
      #exit;
      $buf = <>;# qvs
      ++$lines;
      chomp $buf;
      $qvs{$name} = $buf;
    }
  }
  else{
    printf STDERR ("strange input. $buf\n");
    exit;
  }
}
if($opt_fastq){
  if($lines % 4 != 0){
    printf STDERR ("strange fq. lines % 4 = %d\n", $lines % 4);
  }
}
else{
  if($lines % 2 != 0){
    printf STDERR ("strange fa. lines % 2 = %d\n", $lines % 2);
  }
}

my @names = sort { length($reads{$b}) <=> length($reads{$a}) } keys %reads;

my $maxlength=500;
my $limit=1000000;

for(my $i=1; $i*$windowsize <= $limit; ++$i){
  my $totalbases=0;
  my $threshold = $i*$windowsize;
  for(my $j=0; $j<@names; ++$j){
    if(length($reads{$names[$j]}) >= $threshold){
      $totalbases += length($reads{$names[$j]});
    }
    else{
      last;
    }
  }
  if($totalbases >= $coverage*$estimated_genome_size){
    $maxlength = $threshold;
  }
  else{
    last;
  }
}

if($maxlength+$windowsize > $limit){
  printf STDERR ("WARNING: maybe strange input.\n");
}

for(my $j=0; $j<@names; ++$j){
  if(length($reads{$names[$j]}) >= $maxlength){
    if(!$opt_fastq){
      printf(">%s\n",$names[$j]);
      printf("%s\n",$reads{$names[$j]});
    }
    else{
      printf("\@%s\n",$names[$j]);
      printf("%s\n",$reads{$names[$j]});
      printf("%s\n",$others{$names[$j]});
      printf("%s\n",$qvs{$names[$j]});
    }
  }
  else{
    last;
  }
}

printf STDERR ("%d\n",$maxlength);

