#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define LBUF 2048
#define MAXCOLNUM 32768
#define STRLEN 32768
#define LNAME 256

int r_id=1;

typedef struct base_t{
  char base;
  char qual;
}base_t;

void print_sam(char ** cols, int ncols, char * chrname, base_t * rbuf, base_t * qbuf,int * nnucl);

int main(int argc, char ** argv){
  if(argc != 2){
    fprintf(stderr, "USAGE: <this> <in.vertical|->\n");
    abort();
  }
  char in_vert[LNAME];
  strcpy(in_vert,argv[1]);
  //printf("%s\n",in_vert);

  FILE * vfp;
  if(strcmp("-",in_vert)!=0){
    vfp = fopen(in_vert,"r");
    if(vfp == NULL){
      fprintf(stderr, "cannot open %s\n",in_vert);
      abort();
    }
  }
  else{
    vfp = stdin;
  }
  char * buf = (char*)malloc(sizeof(char)*LBUF);
  if(buf == NULL){
    fprintf(stderr, "cannot allocate memory: buf\n");
    abort();
  }
  char ** cols = (char**)malloc(sizeof(char*)*MAXCOLNUM);
  if(cols == NULL){
    fprintf(stderr, "cannot allocate memory: cols\n");
    abort();
  }
  {
    int i;
    for(i=0; i<MAXCOLNUM; ++i){
      cols[i] = (char*)malloc(sizeof(char)*LBUF);
      if(cols[i] == NULL){
        fprintf(stderr,"cannot allocate memory:cols[%d]\n",i);
        abort();
      }
    }
  }
  char chrname[LNAME];
  chrname[0] = '\0';
  base_t * rbuf = (base_t*)malloc(sizeof(base_t)*STRLEN);
  if(rbuf == NULL){
    fprintf(stderr,"cannot allocate memory: rbuf\n");
    abort();
  }
  base_t * qbuf = (base_t*)malloc(sizeof(base_t)*STRLEN);
  if(qbuf == NULL){
    fprintf(stderr,"cannot allocate memory: qbuf\n");
    abort();
  }
  int * nnucl = (int*)malloc(sizeof(int)*MAXCOLNUM);
  if(nnucl == NULL){
    fprintf(stderr,"cannot allocate memory: nnucl\n");
    abort();
  }

  void chomp(char * str){
    int len = strlen(str);
    if(str[len-1] == '\n')
      str[len-1] = '\0';
    return;
  }

  int i;
  int ncols;
  for(i=0; fgets(buf,LBUF,vfp) != NULL; ++i){
    chomp(buf);
    if(buf[0] == '\0'){
      ncols = i;
      if(i>0)
        print_sam(cols,ncols,chrname,rbuf,qbuf,nnucl);
      i=-1;
      chrname[0] = '\0';
      continue;
    }
    else if(buf[0] == '%'){
      strcpy(chrname,&buf[1]);
      i=-1;
      continue;
    }
    else{
      strcpy(cols[i],buf);
    }
  }
  if(i>0)// final record
    print_sam(cols,ncols,chrname,rbuf,qbuf,nnucl);

  fclose(vfp);
  free(buf);
  for(i=0; i<MAXCOLNUM; ++i){
    free(cols[i]);
  }
  free(cols);
  free(rbuf);
  free(qbuf);
  free(nnucl);

  return 0;
}

void print_record(base_t * rbuf, base_t * qbuf, int samstt, int vertstt, int r_id, char * chrname);
void print_sam(char ** cols, int ncols, char * chrname, base_t * rbuf, base_t * qbuf,int * nnucl){
  int i,j;
  /*
  printf("%%%s\n",chrname);
  for(i=0; i<ncols; ++i){
    printf("%s\n",cols[i]);
  }
  printf("\n");
  return;
  */

  int maxcollen = 0;
  for(i=0; i<ncols; ++i){
    if(maxcollen < strlen(cols[i])){
      maxcollen = strlen(cols[i]);
    }
  }
  for(i=0; i<ncols; ++i){
    int len = strlen(cols[i]);
    for(j=len; j<maxcollen; ++j){
      cols[i][j] = ' ';
    }
    cols[i][j] = '\0';
  }
  maxcollen = (maxcollen-1)/2;
  for(i=0; i<ncols; ++i){
    if(cols[i][0] == ' '){
      fprintf(stderr, "WARNING: no seed base\n");
      fprintf(stderr, "line: %d\n",i+1);
      fprintf(stderr, "chrname: %s\n",chrname);
      //abort();
      return;
    }
    rbuf[i].base = cols[i][0];
    rbuf[i].qual = cols[i][((int)strlen(cols[i])-1)/2+1+0];
  }
  rbuf[i].base = '\0';
  rbuf[i].qual = '\0';
  //strcpy(qbuf,rbuf);
  {
    int j;
    for(j=0; j<=i; ++j){
      qbuf[j].base = rbuf[j].base;
      qbuf[j].qual = rbuf[j].qual;
    }
  }

  if(rbuf[0].base == '-'){
    fprintf(stderr,"souteigai: heading hyphen\n");
    abort();
  }
  else{
    nnucl[0] = 1;
  }
  for(i=1; i<ncols; ++i){
    if(rbuf[i].base == '-'){
      nnucl[i] = nnucl[i-1];
    }
    else{
      nnucl[i] = nnucl[i-1]+1;
    }
  }

  print_record(rbuf,qbuf,1,0,r_id,chrname);
  ++r_id;
  for(j=1; j<maxcollen; ++j){
    int qi=0;
    int samstt;
    int vertstt;
    for(i=0; i<ncols; ++i){
      qbuf[qi].base = cols[i][j];
      qbuf[qi].qual = cols[i][((int)strlen(cols[i])-1)/2+1+j];
      if(qbuf[qi].base == ' ' || qbuf[qi].base == '\t'){
        if(qi>0){
          qbuf[qi].base = '\0';
          qbuf[qi].qual = '\0';
          print_record(rbuf,qbuf,samstt,vertstt,r_id,chrname);
          ++r_id;
          qi=0;
          continue;
        }
        else{
          qi=0;
          continue;
        }
      }
      else{
        if(qi == 0){
          if(rbuf[i].base == '-'){
            // heading '-'s
            samstt = nnucl[i]+1;// 1-origin
          }
          else{
            samstt = nnucl[i];// 1-origin
          }
          vertstt = i;
        }
        ++qi;
        continue;
      }
    }
    if(qi>0){
      qbuf[qi].base = '\0';
      qbuf[qi].qual = '\0';
      print_record(rbuf,qbuf,samstt,vertstt,r_id,chrname);
    }
  }
}

void print_record(base_t * rbuf, base_t * qbuf, int samstt, int vertstt, int r_id, char * chrname){
  printf("%d\t",r_id);
  printf("0\t");
  printf("%s\t",chrname);
  printf("%d\t",samstt);
  printf("255\t");

  // print cigar
  //int len = strlen(qbuf);
  int len; for(len=0; qbuf[len].base != '\0'; ++len);
  int i;
  int prevop = '\0';
  int nop=0;
  for(i=0; i<len; ++i){
    int op;
    if(rbuf[vertstt+i].base == '-' && qbuf[i].base == '-'){
      continue;
    }
    else if(qbuf[i].base == '-' && rbuf[vertstt+i].base != '-'){
      op = 'D';
    }
    else if(rbuf[vertstt+i].base == '-' && qbuf[i].base != '-'){
      op = 'I';
    }
    else{
      op = 'M';
    }
    if(prevop == op){
      ++nop;
    }
    else{
      if(nop > 0){
        printf("%d%c",nop,prevop);
      }
      prevop = op;
      nop=1;
    }
  }
  if(nop > 0){
    printf("%d%c\t",nop,prevop);
  }

  printf("*\t");// RNEXT
  printf("0\t");// PNEXT
  printf("0\t");// TLEN
  int j;
  for(i=0,j=0; i<len; ++i){
    if(qbuf[i].base != '-'){
      qbuf[j].base = qbuf[i].base;
      qbuf[j].qual = qbuf[i].qual;
      ++j;
    }
  }
  qbuf[j].base = '\0';
  qbuf[j].qual = '\0';
  //printf("%s\t", qbuf);
  for(len=0; qbuf[len].base != '\0'; ++len){
    putchar(qbuf[len].base);
  }
  putchar('\t');
  //printf("*\n");
  for(len=0; qbuf[len].qual != '\0'; ++len){
    putchar(qbuf[len].qual);
  }
  putchar('\n');
}
